package de.historischekommission_muenchen.oxydition.brfedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionWithCertaintyBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertFromRegisterIntoLocationWithCertainty;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Eintrag aus der
 * Ortsregisterdatenbank auswählen und als Empfangsort in das
 * <correspAction @type="received"> den <teiHeader> eine Briefes einfügen
 * können. Dabei können sie die Angabe als unsicher markieren.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertMetadataReceivedPlace extends InsertFromRegisterIntoLocationWithCertainty {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen Ortseintrag aus der Registerdatenbank heraussuchen können. "
				+ "Der gewählte Ort wird im Metadatenapparat des TEI-Headers als Empfangsort des Briefes hinterlegt.";
	}

	@Override
	protected void initializeVariables() {
		nodeVal = "//*[local-name()='listPlace']/*[local-name()='place']";
		expressionVal = "//*[local-name()='placeName']";

		dialogTitle = "Metadaten für Empfangsort eingeben";
		xPathLocation = "//correspAction[@type='received']";
	}

	@Override
	protected String constructEntityFragment(ListSelectionBean input) {

		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<placeName " + XMLUtility.TEIXMLNS + " key='" + input.id + "'");

		if (((ListSelectionWithCertaintyBean) input).isUncertain) {
			fragmentBuilder.append(" cert='low'");
		}

		String value = XMLUtility.escapeXML(input.value);
		fragmentBuilder.append(">" + value + "</placeName>");

		return fragmentBuilder.toString();
	}

}
