package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.text.Document;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.DoubleListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.OxyditionInputFilter;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.ReadRegisterStorer;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.SelectionListSearchFieldListener;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.UpdatingJList;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;

/**
 * Dialogfenster für Operationen die zwei, miteinander verknüpfte Auswahllisten
 * benötigen. Die erste Liste, wird bei Fenstererstellung mittels der dem
 * Konstruktor übergebenen Daten befüllt. Wenn die Benutzer*innen einen Eintrag
 * aus dieser Liste markieren und auf den Button "Auswählen" klicken, werden
 * anhand dieser Auswahl und der in einem Datenhaltungsobjekt übergebenen
 * Parametern, die Daten für die zweite Liste ermittelt und in diese eingefügt.
 * Wählt die Benutzer*innen einen Eintrag aus der zweiten Liste aus und beenden
 * die Eingabe mittels des Buttons "Einfügen" werden die Daten der gewählten
 * Einträge in beiden Listen zurückgegeben
 * 
 * @author Maximilian Schrott
 *
 */
public class DoubleListSelectionDialog extends OxyditionOperationDialog {

	private static final long serialVersionUID = 1991948572622254269L;

	private JPanel centerPanel;

	private UpdatingJList<String> listOne;
	private UpdatingJList<String> listTwo;

	private JTextField listOneSearchField;
	private JTextField listTwoSearchField;

	private String[] listOneEintraege;
	private String[] listOneIds;
	private String[] listTwoEintraege;
	private String[] listTwoIds;

	private JButton listOneSelectButton;
	private ReadRegisterStorer listTwoUpdater;

	private static final Dimension LABELMAXSIZE = new Dimension(5000, 20);
	private static final Dimension TEXTFIELDMAXSIZE = new Dimension(5000, 25);

	private float leftAlign = Component.LEFT_ALIGNMENT;
	private int dialogVSize = 750;

	private DoubleListSelectionBean inputBean;

	private boolean usesListOneID;

	/**
	 * Konstruktor ohne ID-Parameter (wird durch leere Liste ersetzt)
	 * 
	 * @param parentFrame      - das Frame des Oxygenfensters
	 * @param title            - Der Titel des Dialogfensters
	 * @param listTwoStorer    - ein ReadRegisterStorer mit den
	 *                         ReadRegisterparametern für die zweite Liste
	 * @param listOneEintraege - die Vorschlagsliste für die erste Liste
	 */
	public DoubleListSelectionDialog(JFrame parentFrame, String title, ReadRegisterStorer listTwoStorer,
			String[] listOneEintraege) {
		super(parentFrame, title);

		this.listOneEintraege = listOneEintraege;
		listOneIds = new String[listOneEintraege.length];
		listTwoUpdater = listTwoStorer;
		usesListOneID = false;
		assembleMainContent();
	}

	/**
	 * Konstruktor mit ID-Parameter
	 * 
	 * @param parentFrame      - das Frame des Oxygenfensters
	 * @param title            - Der Titel des Dialogfensters
	 * @param listTwoStorer    - ein ReadRegisterStorer mit den
	 *                         ReadRegisterparametern für die zweite Liste
	 * @param listOneEintraege - die Vorschlagsliste für die erste Liste
	 */
	public DoubleListSelectionDialog(JFrame parentFrame, String title, ReadRegisterStorer listTwoStorer,
			String[] listOneEintraege, String[] listOneIDs) {
		super(parentFrame, title);

		this.listOneEintraege = listOneEintraege;
		this.listOneIds = listOneIDs;
		listTwoUpdater = listTwoStorer;
		usesListOneID = true;
		assembleMainContent();
	}

	private void assembleMainContent() {
		insertCenterPanel();
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	private void insertCenterPanel() {

		centerPanel = new JPanel();

		listOneSearchField = new JTextField();
		listOne = assembleList(listOneSearchField, listOneEintraege, listOneIds);

		centerPanel.add(DialogUtility.BIGRIGID);
		addListOneSelectButton();
		centerPanel.add(DialogUtility.BIGRIGID);

		// Die zweite Liste zuerst mit leeren Feldern und disabled initialisieren
		listTwoSearchField = new JTextField();
		listTwo = assembleList(listTwoSearchField);
		listTwoSearchField.setEnabled(false);
		listTwo.setEnabled(false);

		mainContent.add(centerPanel, BorderLayout.CENTER);
	}

	private void addListOneSelectButton() {
		listOneSelectButton = new JButton("Auswählen");
		listOneSelectButton.addActionListener(new selectButtonListener());
		centerPanel.add(listOneSelectButton);
	}

	private UpdatingJList<String> assembleList(JTextField searchField, String[] listEintrage, String[] listIds) {

		centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
		addListLabel();
		centerPanel.add(DialogUtility.SMALLRIGID);
		addListSearchField(searchField);
		centerPanel.add(DialogUtility.BIGRIGID);
		UpdatingJList<String> list = addList(listEintrage, listIds);
		linkSearchFieldAndList(searchField, list);

		return list;
	}

	// Methodenvariante für Initialisierung der zweiten Liste
	private UpdatingJList<String> assembleList(JTextField searchField) {
		return assembleList(searchField, new String[0], new String[0]);
	}

	private void addListLabel() {
		JLabel label = new JLabel("Liste filtern:");
		label.setAlignmentX(LEFT_ALIGNMENT);
		label.setMaximumSize(LABELMAXSIZE);
		centerPanel.add(label);
	}

	private void addListSearchField(JTextField searchField) {

		searchField.setAlignmentX(leftAlign);
		searchField.setMaximumSize(TEXTFIELDMAXSIZE);
		OxyditionInputFilter.setForInput(searchField);
		centerPanel.add(searchField);

	}

	private UpdatingJList<String> addList(String[] listEintraege, String[] listIds) {

		UpdatingJList<String> list = new UpdatingJList<String>(listEintraege, listIds);
		JScrollPane listScroll = new JScrollPane();
		listScroll.setViewportView(list);
		listScroll.setAlignmentX(leftAlign);
		listScroll.setMaximumSize(new Dimension(5000, 250));
		centerPanel.add(listScroll);

		return list;
	}

	private void linkSearchFieldAndList(JTextField searchField, UpdatingJList<String> list) {
		Document searchFieldDocument = searchField.getDocument();
		searchFieldDocument.addDocumentListener(new SelectionListSearchFieldListener(list));

	}

	public DoubleListSelectionBean displayDialog() {

		inputBean = new DoubleListSelectionBean();
		okButton.addActionListener(new DoulbeListOkButtonListener());

		readyDialog();
		return inputBean;
	}

	private void readyDialog() {
		getContentPane().add(mainContent);
		pack();
		setSize(950, dialogVSize);
		setLocationRelativeTo(getOwner());
		setVisible(true);
	}

	@Override
	protected void cancelButtonAction() {
		dispose();
	}

	protected class selectButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			if (listOne.getSelectedIndex() > -1) {
				inputBean.listOneValue = listOne.getSelectedValue();
				inputBean.listOneID = listOne.getSelectedMetaEntry();

				fillListTwo();
			}
		}
	}

	private void fillListTwo() {

		String urlVal;

		if (usesListOneID) {
			urlVal = inputBean.listOneID;
		} else {
			urlVal = inputBean.listOneValue;
		}

		try {
			ReadRegisterBasic listTwoRegisterReader;

			listTwoRegisterReader = listTwoUpdater.getListData(urlVal);

			listTwoEintraege = listTwoRegisterReader.getEintrag();

			if (listTwoEintraege.length > 0) {
				String[] listTwoShortenedEintraege = shortenProposals(listTwoRegisterReader.getEintrag());

				listTwoIds = listTwoRegisterReader.getID();
				listTwo.fillList(listTwoShortenedEintraege, listTwoIds);

				listTwoSearchField.setEnabled(true);
				listTwo.setEnabled(true);
				listTwoSearchField.setText("");
			} else {
				listTwoSearchField.setEnabled(false);
				listTwo.setEnabled(false);
				listTwoSearchField.setText("Keine Einträge");
			}

		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(getParent(), e);
		}
	}

	private String[] shortenProposals(String[] eintrag) {

		String[] shortendProposals = new String[eintrag.length];

		for (int i = 0; i < eintrag.length; i++) {
			String fullText = eintrag[i];

			if (fullText.length() > 175) {
				String shortendText = fullText.substring(0, 174);
				shortendText += "...";
				shortendProposals[i] = shortendText;
			} else {
				shortendProposals[i] = fullText;
			}
		}
		return shortendProposals;
	}

	public class DoulbeListOkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {

			if (listTwo.getSelectedIndex() > -1) {
				extractListTwoInput();
				dispose();
			}
		}
	}

	private void extractListTwoInput() {

		inputBean.listTwoValue = listTwo.getSelectedValue();
		inputBean.listTwoID = listTwo.getSelectedMetaEntry();
		inputBean.isFilled = true;
	}
}
