package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;

/**
 * Vorlage für Dialogfenster, die Strukturelemente mit einer dokumentenabhänigen
 * ID in ein Dokument einfügen. Die Vorlage ist darauf ausgelegt, durch erbende
 * Klassen in Aufbau und Funktion an die jeweiligen Anforderungen angepasst zu
 * werden.
 * 
 * @author Maximilian Schrott
 *
 */

public class InsertStructureElementDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = -4433736433653098758L;

	protected JTextField elementNumField;
	protected JTextField normNumField;
	protected JTextField titleField;
	protected InsertStructureElementBean inputBean;
	protected boolean numberNeeded;

	public InsertStructureElementDialog(JFrame owner, String dialogTitle, boolean numberNeeded) {
		super(owner, dialogTitle);

		this.numberNeeded = numberNeeded;

		String elementNumFieldLabelText = createElementNumFieldLabelText(numberNeeded);

		elementNumField = addField(elementNumFieldLabelText);
		normNumField = addField("Nummer (arabische Ziffern/Großbuchstaben)");
		titleField = addField("Überschrift");

		insertOperationEndButtonPanel("Übernehmen", "Abbrechen");
		okButton.addActionListener(new OkButtonListener());
	}

	private String createElementNumFieldLabelText(boolean numberNeeded) {

		if (numberNeeded) {
			return ("Nummer");
		} else {
			return ("Nummer (optional)");
		}
	}

	public InsertStructureElementBean displayDialog() {

		inputBean = new InsertStructureElementBean();

		readyDialog();
		return inputBean;
	}

	protected class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}
		}
	}

	protected boolean validateInput() {

		boolean allValid = true;
		String notAllValidMessage = "";

		// Prüfen ob Nummernfelder ausgefüllt sind, falls eine Nummer benötigt
		// wird
		if (numberNeeded) {

			if (!DialogUtility.isFilledTextfield(elementNumField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die Nummer eingegeben;\n";
			}
		}

		if (!DialogUtility.isFilledTextfield(normNumField)) {
			allValid = false;
			notAllValidMessage += "Kein Wert für die TOP-Nummer (arabische Ziffern/Großbuchstaben) eingegeben;\n";
		} else if (!isValidNormNumber(normNumField)) {
			allValid = false;
			notAllValidMessage += "Wert für Nummer (arabische Ziffern/Großbuchstaben) ungültig. Keine Zeichen außer den genannten sind erlaubt.";
		}

		if (!DialogUtility.isFilledTextfield(titleField)) {
			allValid = false;
			notAllValidMessage += "Kein Wert für Überschrift eingegeben; \n";
		}

		if (allValid) {
			return true;
		} else {
			displayInputNotAllValidMessage(notAllValidMessage);
			return false;
		}
	}

	protected void extractInput() {
		inputBean = new InsertStructureElementBean();

		inputBean.elementNum = elementNumField.getText();
		inputBean.normNum = normNumField.getText();
		inputBean.title = titleField.getText();
		inputBean.isFilled = true;
	}

	// Überprüft ob die maschinenlesbare TOP-Nummer ein gültiges Format
	// besitzt
	private boolean isValidNormNumber(JTextField normNumField) {

		String normNumber = normNumField.getText();
		return normNumber.matches("[A-Z]+|\\d+");
	}
}
