package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;

public abstract class OxyditionOperationDialog extends JDialog {

	private static final long serialVersionUID = -999118474407108114L;

	protected JButton okButton;
	protected JButton cancelButton;
	protected JPanel mainContent;
	protected JPanel operationEndButtonPanel;

	public OxyditionOperationDialog(JFrame parentFrame, String title) {
		super(parentFrame, title, true);
		mainContent = new JPanel(new BorderLayout(10, 10));
		mainContent.setBorder(new EmptyBorder(10, 10, 10, 10));
	}

	private void setUpEndButtonKeyBehavior() {
		rootPane.setDefaultButton(okButton);
		setUpEscapeCancelBehavior();
	}

	private void setUpEscapeCancelBehavior() {
		int escapeKeyEvent = KeyEvent.VK_ESCAPE;

		KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(escapeKeyEvent, 0);
		rootPane.registerKeyboardAction(new EscapeListener(), escapeKeyStroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	protected void insertOperationEndButtonPanel(String okButtonText, String cancelButtonText) {
		operationEndButtonPanel = assembleOperationEndButtonPanel(okButtonText, cancelButtonText);
		mainContent.add(operationEndButtonPanel, BorderLayout.PAGE_END);
	}

	protected JPanel assembleOperationEndButtonPanel(String okButtonText, String cancelButtonText) {

		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER, 10, 4));

		okButton = new JButton(okButtonText);
		panel.add(okButton);

		cancelButton = new JButton(cancelButtonText);
		cancelButton.addActionListener(new CancelButtonListener());
		panel.add(cancelButton);

		setUpEndButtonKeyBehavior();

		return panel;
	}

	protected abstract void cancelButtonAction();

	private class CancelButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {
			cancelButtonAction();
		}
	}

	private class EscapeListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			cancelButton.doClick();
		}
	}

	/**
	 * Standartisiertes Fehlerverhalten bei BadLocationExceptions: Bringt den
	 * Stacktrace auf die Konsole und öffnet eine JOptionPane mit einer
	 * Fehlermeldung für die Bearbeiter*innen
	 * 
	 * @param e - die abgefangene BadLocationException
	 */
	protected void displayBadLocationMessage(BadLocationException e) {

		String badLocationTitle = "Schwerer Fehler";
		String badLocationMessage = "Schwerer Fehler: BADLOCATIONEXCEPTION";

		e.printStackTrace();
		JOptionPane.showMessageDialog(getParent(), badLocationMessage, badLocationTitle, JOptionPane.WARNING_MESSAGE);

	}

	/**
	 * Standartisiertes Verhalten, wenn eine Benutzereingabe nicht vollständig oder
	 * bestimmte Eingabe ungültig sind: Öffnet ein JOptionPane, das den
	 * Benutzer*innen eine übergebene Nachricht anzeigt.
	 * 
	 * @param notAllValidMessage - Die Fehlermeldung die in der JOptionPane
	 *                           angezeigt werden soll
	 */
	protected void displayInputNotAllValidMessage(String notAllValidMessage) {
		JOptionPane.showMessageDialog(getParent(), notAllValidMessage, "Eingabe unvollständig oder ungültig",
				JOptionPane.WARNING_MESSAGE);
	}
}
