package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;

/**
 * Spezialisierung von AutoTagOperation, für die halbautomatische Referenzierung
 * auf die Registerdatenbank für Sachschlagwörter
 * 
 * @author Maximilian Schrott
 *
 **/

public class AutoTagKeywordOperation extends AutoTagOperation {

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	public String getDescription() {
		return "Die Benutzer*innen markieren einen Abschnitt des Texts und rufen diese Operation aus."
				+ " Die Operation auf eine Registerdatenbank für Sachschlagwörter zurück und zeigt bestimmte"
				+ " Einträge daraus als Vorschläge für die Benutzer*innen an. Gleichzeitig wird das Dokument nach allen"
				+ " Vorkommnissen des markierten Textausschnitts durchsucht. Die Benutzer*innen wählen einen der vorgeschlagenen Einträge aus"
				+ " der Registerdatenbank aus und entscheidet bei jedem Suchtreffer, ob es sich tatsächlich um ein"
				+ " Vorkommen des gesuchten Schlagworts oder einen flaschen Treffer handelt. In letzterem Fall wird"
				+ " der Treffer übersprungen. Andernfalls können die Benutzer*innen die Textstelle für die Referenzierung vormerken."
				+ " Beim erfolgreichen Abschluss der Operation werden dann alle vorgemerkten Treffer mit <rs>-Elementen umgeben,"
				+ " die auf den ausgewählten Eintrag in der Registerdatenbank für Saschschlagwörter verweisen.";
	}

	@Override
	protected boolean validateArguments(ArgumentsMap args) {

		return validateUrlArgument(args);
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);
		return new KeywordReadRegister(urlArgString);
	}

	@Override
	protected Object[] collectTypeArguments(ArgumentsMap args) {

		Object[] typeArguments = new Object[4];
		typeArguments[1] = "Sachschlagwort";

		return typeArguments;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
