package de.historischekommission_muenchen.oxydition.common.operations;

import java.util.regex.Pattern;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionTextfieldBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertFromRegisterWithTextfield;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen eine Literaturangabe (als
 * <bibl>-Element) einfügen können. Die Angabe ist mit der
 * Literaturregisterdatenbank verknüpft und übernimmt den dort hinterlegten
 * Kurztitel. Außerdem haben die Benutzer*innen die Möglichkeit die genauen
 * Textstelle (z.B. die Seitenzahl) einzugeben, die dann zur Literaturangabe
 * hinzugefügt wird.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertLiteratureInfo extends InsertFromRegisterWithTextfield {

	@Override
	public String getDescription() {

		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen eine Literaturangabe aus dem Register auswählen und den genauen Fundort eingeben können."
				+ "Der Kurztitel des Werks wird dann zusammen mit der Fundstelle an der Cursorposition in das Dokument eingefügt";
	}

	@Override
	protected void initializeVariables() {
		nodeVal = "//*[local-name()='listBibl']/*[local-name()='bibl']";
		expressionVal = "//*[local-name()='title' and @type='Volltitel']";
		idVal = "@xml:id + \"$#\" + "
				+ "//*[local-name()='title' and @type='Kurztitel']/*[local-name()='seg' and @type='Kennabschnitt'] + \"$#\" + "
				+ "//*[local-name()='title' and @type='Kurztitel']/*[local-name()='seg' and @type='Titelabschnitt']";
		dialogTitle = "Literaturangabe einfügen";
		textFieldLabel = "Genaue Textstelle, z.B, Seitenzahl (optional)";
	}

	@Override
	protected String constructEntityFragment(ListSelectionTextfieldBean input) {

		String selectedValue = input.id;
		String[] selectedSplit = selectedValue.split(Pattern.quote("$#"));

		String id = selectedSplit[0];
		String referenceSegment = XMLUtility.escapeXML(selectedSplit[1]);

		String mainSegment = "";
		String scope = XMLUtility.escapeXML(input.textInput);

		if (selectedSplit.length > 2) {
			mainSegment = XMLUtility.escapeXML(selectedSplit[2]);
		}

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<bibl " + XMLUtility.TEIXMLNS + " corresp='" + id + "'>");
		fragmentBuilder.append("<title>");

		if (!referenceSegment.isEmpty()) {
			fragmentBuilder.append("<hi>" + referenceSegment + "</hi>");
		}
		fragmentBuilder.append(mainSegment + "</title>");
		if (!scope.isEmpty()) {
			fragmentBuilder.append(" <biblScope>" + scope + "</biblScope>");
		}
		fragmentBuilder.append("</bibl>");

		return fragmentBuilder.toString();

	}

}
