package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * 
 * Oxygen-Author-Operation mit der die Benutzer*innen dem XML-Element an der
 * Position des Textcursors ein Attribut hinzufügen können, dass auf einen
 * Eintrag in der Registerdatenbank verweist. Die RDB-Pfade und der
 * Attributsname werden durch Operationsargumenten festgelegt.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertReferenceAttribute extends ReadRegisterOperation {

	private static String ARGUMENT_ATTRIBUTE = "Attribut";

	private static String attributeArgDescription = "Das Attribut in dem die ID des referenzierten \" + \"RDB-Eintrags gespeichert wird. Z.B.: corresp";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription),
			new ArgumentDescriptor(ARGUMENT_NODE, ArgumentDescriptor.TYPE_STRING, nodeArgDescription),
			new ArgumentDescriptor(ARGUMENT_EXPRESSION, ArgumentDescriptor.TYPE_STRING, expressionArgDescription),
			new ArgumentDescriptor(ARGUMENT_ATTRIBUTE, ArgumentDescriptor.TYPE_STRING, attributeArgDescription)

	};

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die festgelegten Basisknoten aus der festgelegten Dati in der festgelegten"
				+ " Vorschlagsform den Benutzer*innen zur Auswahl gestellt werden. Wählen sie einen Eintrag aus, dann fügt die"
				+ " Operation eine Referenz auf diesen Eintrag als Attibut vom festgelegten Typ in den Knoten an der Position des Textcursors ein";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			String surroundNodeText = surroundNode.getTextContent();

			ReadRegisterBasic registerReader = getRegisterEntries(args);
			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			String frameName = "Element mit Registerdatenbank verknüpfen";

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, frameName, eintraege, ids);
			dlg.updateSearchRegisterField(surroundNodeText);
			ListSelectionBean input = dlg.displayDialog();
			String attribute = (String) args.getArgumentValue(ARGUMENT_ATTRIBUTE);

			if (input.isFilled) {
				AttrValue attributeValue = new AttrValue(input.id);
				documentController.setAttribute(attribute, attributeValue, surroundNode);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	private boolean validateArguments(ArgumentsMap args) {
		boolean allValid = true;

		allValid = validateBaseArguments(args);

		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_ATTRIBUTE))) {
			allValid = false;
		}

		return allValid;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
