package de.historischekommission_muenchen.oxydition.common.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.InsertStructureElementDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertStructureElement;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Unterabschnitt ins
 * Dokument einfügen können. Hierfür müssen sie in einem Dialogfenster die
 * Unterabschnittsnummer eingeben. Ein Unterabschnitt ist ein <div> Element mit
 * type-Attribut="Unterabschnitt" und einer xml:id, die sich aus der
 * Dokumenten-ID und der Unterabschnittnummer ergibt.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertSubSection extends InsertStructureElement {

	@Override
	public String getDescription() {

		return "Ermöglicht es den Benutzer*innen einen unterabschnitt in einen Textabschnitt einer Edition einzufügen.\n"
				+ " Unterabschnitte sind größere im Inhaltsverzeichnis aufgeführte Texteinheiten mit Überschrift.";
	}

	@Override
	protected InsertStructureElementBean getInputData(JFrame oxygenFrame, String dialogTitle) {

		InsertStructureElementDialog dlg = new InsertStructureElementDialog(oxygenFrame, "Unterabschnitt einfügen",
				true);
		InsertStructureElementBean input = dlg.displayDialog();

		return input;
	}

	@Override
	protected String constructTopicXMLFragment(InsertStructureElementBean input, String idPrefix) {

		String normNum = normaliseNormNum(input.normNum);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<div " + XMLUtility.TEIXMLNS + " ");
		fragmentBuilder.append("type='Unterabschnitt' ");
		fragmentBuilder.append("xml:id='" + idPrefix + "g" + normNum + "'>");
		fragmentBuilder.append(constructHeadXMLFragment(input));
		fragmentBuilder.append("</div>");

		return fragmentBuilder.toString();
	}

	@Override
	protected String constructHeadXMLFragment(InsertStructureElementBean input) {

		String elementNum = XMLUtility.escapeXML(input.elementNum);

		StringBuilder headBuilder = new StringBuilder();
		headBuilder.append("<head " + XMLUtility.TEIXMLNS + ">");
		headBuilder.append("<num " + XMLUtility.TEIXMLNS + ">" + elementNum + "</num>");
		headBuilder.append(input.title + "</head>");

		return headBuilder.toString();

	}

	@Override
	protected boolean isCaretPositionValid(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {
		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

		return XMLUtility.elementTypeEquals(surroundNode, "Textabschnitt")
				|| XMLUtility.elementTypeEquals(surroundNode, "Quelldokument");

	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf kein Unterabschnitt eingefügt werden.\n"
				+ "Fügen Sie einen Unterabschnitt nur auf der obersten Gliederungsebene eines Textabschnitts ein.";

	}

	@Override
	protected String findIdPrefix(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {

		// Da Unterabschnitte nur auf oberster Gliederungsebene erlaubt sind,
		// ist immer die ProtokollId das passende Präfix
		AttrValue protIdVal = ProtocolUtility.getProtocolID(documentController);

		return protIdVal.getRawValue();
	}

}
