package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Spezialisierung von TagSelectionOperation, für Operationen, die auf die
 * Registerdatenbank für Sachschlagwörter verweisen.
 * 
 * @author Maximilian Schrott
 * 
 */
public class TagKeywordOperation extends TagSelectionOperation {

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen die markierte Textstelle mit einem auswählbaren Eintrag"
				+ " im Sachschlagwortsregister zu verknüpfen";
	}

	@Override
	protected boolean validateArguments(ArgumentsMap args) {

		return validateUrlArgument(args);
	}

	@Override
	protected void tagSelection(ListSelectionBean input, String typeArgVal) throws AuthorOperationException {
		int selectionStart = editorAccess.getSelectionStart();
		int selectionEnd = editorAccess.getSelectionEnd();

		String tagFragment = constructTagFragment(input, "Sachschlagwort");
		documentController.surroundInFragment(tagFragment, selectionStart, selectionEnd - 1);
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);
		return new KeywordReadRegister(urlArgString);

	}

	protected String getFrameName() {
		return "Auswahl als Sachschlagwort markieren";
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
