package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Abstrakte Oberklasse für Oxygen-Author-Operationen, mit denen die Benutzer*innen
 * einen Verweise auf einen RDB-Eintrag an einer voreingestellten Position des
 * Dokuments einfügen können. Der Einfügeort, die Pfadbeschreibungen zu den
 * RDB-Einträgen und die Form des eingefügten XML-Fragments müssen von den
 * erbenden Klassen festegelegt werden.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertFromRegisterIntoLocation extends ReadRegisterOperationJustURL {

	// Weitere Operationsvariablen
	protected String xPathLocation;
	protected String dialogTitle;

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		initializeVariables();

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			ReadRegisterBasic registerReader = getRegisterEntries(args);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, dialogTitle, eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertFromRegister(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	protected void insertFromRegister(ListSelectionBean input) throws AuthorOperationException {
		String entityFragment = constructEntityFragment(input);
		documentController.insertXMLFragmentSchemaAware(entityFragment, xPathLocation,
				AuthorConstants.POSITION_INSIDE_LAST);
	}
	
	protected abstract String constructEntityFragment(ListSelectionBean input);

}
