package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;

/**
 * Abstrakte Oberklasse Operationen, mit denen die Benutzer*innen Strukturelemente mit
 * einer, aus der DokumentenID, abgeleiteten xml:id einfügen können. Gibt den
 * Operationsablauf in doOperation() vor, die Form der Einfügung wird durch die
 * erbenden Klassen mittels der bereitgestellten abstrakten Methoden angepasst.
 * 
 * @author Maximilian Schrott
 * 
 */
public abstract class InsertStructureElement extends OxyditionOperation {

	@Override
	public abstract String getDescription();

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		InsertStructureElementBean inputBean = new InsertStructureElementBean();

		AttrValue protocolID = ProtocolUtility.getProtocolID(documentController);

		if (protocolID == null) {
			JOptionPane.showMessageDialog(oxygenFrame,
					"Der Dokumentenkopf muss eingefügt werden, bevor ein Strukturelement hinzugefügt werden kann",
					"Dokuemntenkopf noch nicht eingefügt", JOptionPane.WARNING_MESSAGE);
			return;
		}

		if (protocolID != null) {

			try {

				boolean validSurroundNode = isCaretPositionValid(caretPosition, documentController);

				if (!validSurroundNode) {

					displayInvalidCaretPositionMessage();

				} else {

					String idPrefix = findIdPrefix(caretPosition, documentController);

					inputBean = getInputData(oxygenFrame, "Tagesordnungspunkt einfügen");

					if (inputBean.isFilled) {

						String topicFragment = constructTopicXMLFragment(inputBean, idPrefix);
						documentController.insertXMLFragment(topicFragment, caretPosition);
					}
				}

			} catch (BadLocationException e) {
				displayBadLocationMessage(e);
			}
		}
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	// Wenn Top-Nummer einstellige Zahl: 0 vorne anfügen
	protected String normaliseNormNum(String normNum) {

		if (normNum.length() == 1) {
			if (!normNum.matches("[A-Z]")) {
				normNum = "0" + normNum;
			}
		}

		return normNum;

	}

	/**
	 * Zeigt das Dialogfenster an, nimmt die Benutzereingaben entgegen und gibt sie
	 * zurück
	 */
	protected abstract InsertStructureElementBean getInputData(JFrame oxygenFrame, String dialogTitle);

	/**
	 * Stellt das XML-Fragment für den einzufügenden Tagesordnungspunkt mit
	 * ausführlichem Header zusammen
	 */
	protected abstract String constructTopicXMLFragment(InsertStructureElementBean inputBean, String idPrefix);

	/**
	 * Konsturiert abhängig von den Benutzereingaben das <head>-Element des TOPs
	 */
	protected abstract String constructHeadXMLFragment(InsertStructureElementBean input);

	/**
	 * Überprüft ob sich der Textcursor an eine für die Einfügeoperation gültigen
	 * Stelle bedindet
	 */
	protected abstract boolean isCaretPositionValid(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException;

	/**
	 * Ermittelt das Präfix für die @xml:id des einzufügenden Elemetns fest. Da die
	 * Funktion erst nach Prüfung der Eingabeposition ausgelöst wird, kann von einer
	 * gültigen Textcursor-Position ausgegangen werdeb
	 */
	protected abstract String findIdPrefix(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException;

}
