package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.IOException;

import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.UnpublishedSourceReadRegister;
import ro.sync.ecss.extensions.api.ArgumentsMap;

/**
 * Abstrakte Oberklasse für Oxygen-Author-Operationen, in denen ein ReadRegister
 * zum Einsatz kommt. Stellt einen einheitlichen Zugriff auf die
 * Readregister-Objekt zur Verfügung. Außerdem werden hier die Pfadargumente für
 * den RDB-Zugriff über ReadRegister initialisiert. Erbende Klassen können diese
 * (und zusätzliche) Argumente benutzen, indem sie sie in ein
 * ArgumentDescritor-Element[] integrieren. Oder sie verzichten auf einzelne
 * Argumente und legen ihren Wert auf Java-Ebene fest.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class ReadRegisterOperation extends OxyditionOperation {

	protected static String ARGUMENT_URL = "URL";
	protected static String ARGUMENT_NODE = "Basisknoten";
	protected static String ARGUMENT_EXPRESSION = "Vorschlag";

	protected static String urlArgDescription = "Die URL der Registerdatenbanksdatei,"
			+ "\nz.B.: \"file:${pd}\\Bearbeitung\\RDB\\${pb}RDB_PER.xml";
	protected static String nodeArgDescription = "Die XPath-Beschreibung des Knotentypen, der refernziert werden soll,"
			+ "z.B. \"//*[local-name()='listPerson']/*[local-name()='person']\"";
	protected static String expressionArgDescription = "Die Form des Vorschlags, der den Benutzer*innen in der Registerliste angezeigt wird."
			+ "\n Muss aus Subelementen der oben ausgewählten Basisknoten bestehen."
			+ "\n Strings stehen ind \"\", Attribute beginnen mit @, Elemente mit /, Subelemente mit // und XPath-Ausdrücke mit .),"
			+ "\n \"//*[local-name()='persName' and type='Registername'] + \" \" +//*[local-name()='note' and @type='Qualifikator']\"";

	protected String idVal = "@xml:id";

	protected boolean validateBaseArguments(ArgumentsMap args) {

		boolean allValid = true;

		allValid = validateUrlArgument(args);

		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_NODE))) {
			allValid = false;
		}
		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_EXPRESSION))) {
			allValid = false;
		}

		return allValid;
	}

	protected boolean validateUrlArgument(ArgumentsMap args) {

		Object urlArgVal = args.getArgumentValue(ARGUMENT_URL);
		return OperationUtility.isValidArgument(urlArgVal);
	}

	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);

		if (urlArgString.contains("SSW")) {
			return new KeywordReadRegister(urlArgString);
		} else if (urlArgString.contains("UGQ")) {
			return new UnpublishedSourceReadRegister(urlArgString);
		} else {
			String nodeArgString = (String) args.getArgumentValue(ARGUMENT_NODE);
			String expressionArgString = (String) args.getArgumentValue(ARGUMENT_EXPRESSION);

			return new StandardReadRegister(urlArgString, nodeArgString, expressionArgString, idVal);
		}
	}

	protected void displayRegisterNotFoundMessage() {

		String RegisterNotFoundTitle = "Zugriffsfehler";
		String RegisterNotFoundMessage = "Zugriff auf Registerdatenbank fehlgeschlagen.\n"
				+ "Möglichweise ist die gesuchte Registerdatenbanksdatei nicht vorhanden oder der Dateipfad in der Operation falsch konfiguriert.";
		JOptionPane.showMessageDialog(oxygenFrame, RegisterNotFoundMessage, RegisterNotFoundTitle,
				JOptionPane.WARNING_MESSAGE);
	}
}
