package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;

/**
 * Abstrakte Oberklasse für Oxygen-Author-Operationen, die auf die
 * Registerdatenbank zugreifen und für denen nur die RDB-Dokuments-URL ein
 * flexibler Parameter ist. Die anderen ReadRegisterparameter werden von den
 * erbenden Klassen statisch festgelegt.
 */
public abstract class ReadRegisterOperationJustURL extends ReadRegisterOperation {

	// Operationsvariablen
	protected String nodeVal;
	protected String expressionVal;

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription), };

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {
		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);

		return new StandardReadRegister(urlArgString, nodeVal, expressionVal, idVal);
	}

	/**
	 * Methode zur Konstruktion des einzufügenden XML-Fragments
	 * 
	 * @param input
	 *            - Datenhaltungsobjekt mit den Eingabedaten
	 * @return Das konstruierte XML-Fragment
	 */
	protected boolean validateArguments(ArgumentsMap args) {
		return validateUrlArgument(args);
	}

	/**
	 * Methode zum Besetzen der Operationsvariablen
	 */
	protected abstract void initializeVariables();
}
