
package de.historischekommission_muenchen.oxydition.common.utilities;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.codec.binary.Base64;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Eine Anwendungs-Klasse zum Durchsuchen und Auswerten der XML-Registerdatei
 * für Sachschlagwörter nach dem Muster der Proteditions. Der Klasse
 * org.bbaw.telote.ediarum.ReadRegister nachempfunden, kann aber
 * Sachschlagwörter auf mehereren Schachetlungsebenen auswerten
 * 
 * @author Maximilian Schrott
 *
 */

public class KeywordReadRegister extends ReadRegisterBasic {

	/**
	 * Konstruktor
	 * 
	 * @param indexURI - URI der Registerdatei
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 */
	public KeywordReadRegister(String indexURI)
			throws ParserConfigurationException, IOException, SAXException, XPathExpressionException {

		// Zur Auswertung der XML-Datei vorbereiten
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
		domFactory.setNamespaceAware(false);
		DocumentBuilder builder = domFactory.newDocumentBuilder();
		// Wenn es sich um eine URL mit Authentifizierung handelt, ..
		InputStream is;
		if (indexURI.indexOf('@') > -1) {
			// .. werden die Verbindungsdaten gelesen ..
			String authString = indexURI.substring(indexURI.indexOf("://") + 3, indexURI.indexOf('@'));
			String webPage = indexURI.substring(0, indexURI.indexOf("://") + 3)
					+ indexURI.substring(indexURI.indexOf('@') + 1);
			byte[] authEncBytes = Base64.encodeBase64(authString.getBytes());
			String authStringEnc = new String(authEncBytes);
			// .. und eine Verbindung mit Login geöffnet.
			URL url = new URL(webPage);
			URLConnection urlConnection = url.openConnection();
			urlConnection.setRequestProperty("Authorization", "Basic " + authStringEnc);
			is = urlConnection.getInputStream();
		} else {
			// Im anderen Fall wird direkt eine Verbindung geöffnet.
			URL url = new URL(indexURI);
			URLConnection urlConnection = url.openConnection();
			is = urlConnection.getInputStream();
		}

		// Dann wird die Datei gelesen
		InputSource inputSource = new InputSource(is);
		Document indexDoc = builder.parse(inputSource);
		// Die xPath-Routinen werden vorbereitet
		XPath xpath = XPathFactory.newInstance().newXPath();

		// Querytext zum finden der Quellen-Einträge
		String nodeQuery = "//*[local-name()='list' and @type='Sachschlagwörter']/*[local-name()='item']";

		// Das XPath-Query wird definiert.
		XPathExpression expr = xpath.compile(nodeQuery);

		// Die Resultate werden ausgelesen
		Object result = expr.evaluate(indexDoc, XPathConstants.NODESET);
		NodeList registerNodes = (NodeList) result;
		// .. dann werden für die Einträge und IDs entsprechend lange
		// Arrays
		// angelegt.
		eintrag = new String[registerNodes.getLength()];
		id = new String[registerNodes.getLength()];

		// Auswertung jedes einzelnen Quelleneintrags
		for (int i = 0; i < registerNodes.getLength(); i++) {

			// das Quellen-<item-selbst
			Element currentElement = (Element) registerNodes.item(i);

			// hinterlegte ID speichern
			id[i] = currentElement.getAttribute("xml:id");

			/*
			 * Anzeigetext aus Quellen-Identifieren und Kennnamen der übergeordneten
			 * Sammlungen erzeugen
			 */
			String eintragString = generateDisplayString(currentElement);

			eintrag[i] = eintragString;
		}
	}

	// Erzeugt aus dem übergebenen <item>-Element, den Vorschlagstext für die
	// Anzeige
	private String generateDisplayString(Element currentElement) {

		StringBuilder displayStringBuilder = new StringBuilder();
		displayStringBuilder.append(constructKeywordLevelFragment(currentElement));
		displayStringBuilder.append(getKeywordTerm(currentElement));

		return displayStringBuilder.toString();
	}

	// Überprüft auf welcher Sachschlagwortschachtelungsebene sich das aktuelle
	// SSW befindet und gibt ein Präfix mit entsprechend vielen
	// Gliederungsstrichen zurück
	private Object constructKeywordLevelFragment(Element currentElement) {

		Element parentList = (Element) currentElement.getParentNode();
		String parentListLevel = parentList.getAttribute("n");
		int parenListLevelInteger = Integer.parseInt(parentListLevel);
		StringBuilder keywordLevelFragment = new StringBuilder("");
		for (int i = 0; i < parenListLevelInteger; i++) {
			keywordLevelFragment.append("- ");
		}

		return keywordLevelFragment;
	}

	// Ermittelt das Sachschlagwort selbst
	private Object getKeywordTerm(Element currentElement) {

		NodeList currentChildren = currentElement.getChildNodes();

		String term = "";

		for (int i = 0; i < currentChildren.getLength(); i++) {

			if (currentChildren.item(i).getNodeType() == Node.ELEMENT_NODE) {
				Element child = (Element) currentChildren.item(i);

				if (child.getNodeName().equals("term")) {
					term = child.getTextContent();
				}
			}
		}
		return term;
	}
}
