package de.historischekommission_muenchen.oxydition.common.utilities;

import java.awt.Component;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

/**
 * Stellt Methoden zur Verfügung, die bei der Ausführung von Author-Operationen
 * helfen
 * 
 * @author Maximilian Schrott
 *
 */
public class OperationUtility {

	/**
	 * Überprüft ob das übergebene Argumentobjekt gültig und verwehrtbar ist
	 * 
	 * @param argVal - Ein Funktionsargument einer AuthorOperation
	 * 
	 */
	public static boolean isValidArgument(Object argVal) {

		return (argVal != null && argVal instanceof String && ((String) argVal).length() > 0);
	}

	/**
	 * Isoliert anhand eines regulären Ausdrucks eine Teilstring aus dem übergebenen
	 * String und gibt diesen zurück
	 * 
	 * @param text    - Der zu untersuchende String
	 * @param pattern - Ein regulärer Ausdruck, muss genau eine Capturing Group (in
	 *                runden Klammern) enthalten
	 * @return - gibt den geufndenen Teilstring zurück oder den Ausgangsstring,
	 *         falls der reguläre Ausdruck kein Match finden konnte
	 */
	public static String subStringWithRegex(String text, String pattern) {

		String subString = text;

		Pattern subStringPattern = Pattern.compile(pattern);
		Matcher subStringMatcher = subStringPattern.matcher(text);

		if (subStringMatcher.find()) {
			subString = subStringMatcher.group(1);
		}

		return subString;
	}

	public static void handleGenericException(Component parentFrame, Exception e) {
		e.printStackTrace();
		displayExceptionMessage(parentFrame, e.getMessage());
	}

	private static void displayExceptionMessage(Component parentFrame, String exceptionMessage) {
		String exceptionTitle = "Ausnahmefehler";
		JOptionPane.showMessageDialog(parentFrame, exceptionMessage, exceptionTitle, JOptionPane.WARNING_MESSAGE);
	}
}
