package de.historischekommission_muenchen.oxydition.protedition.common.dialogs;

import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.SingleInputDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;

/**
 * Dialogfenster für die Operation zum Einfügen von Amerkungsreferenzen mittels
 * der Anmerkungs-ID. Enthält ein Textfeld für die xml:id der referenzierten
 * Anmerkung, das die Eingabe darauf prüft, ob sie die gültige Form einer
 * Anmerkungs-ID hat.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertNoteIDReferenceDialog extends SingleInputDialog {

	private static final long serialVersionUID = -2705682832896775704L;

	public InsertNoteIDReferenceDialog(JFrame owner, String title) {
		super(owner, title, "Fußnoten-XML:ID");
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	@Override
	protected boolean validateInput() {

		boolean allValid = true;
		String notAllValidMessage = "";

		if (!DialogUtility.isFilledTextfield(singleInputField)) {
			allValid = false;
			notAllValidMessage += "Kein Wert für Anmerkungs-ID eingeben";
		} else if (!isValidNoteID(singleInputField)) {
			allValid = false;
			notAllValidMessage += "Wert für Anmerkungs-ID ungültig";
		}

		if (allValid) {
			return true;
		} else {
			displayInputNotAllValidMessage(notAllValidMessage);
			return false;
		}
	}

	// Überprüft ob eingebene AnmerkungsID dem Muster
	// nt-DOKUMENTENKÜRZEL_DREITEILIGERIDSUFFIX entspricht
	private boolean isValidNoteID(JTextField noteIdInput) {

		String noteId = noteIdInput.getText();

		return noteId.matches("nt-[^_]{9,10}_[^_]{3}_[^_]{3}_[^_]{3}");
	}
}
