package de.historischekommission_muenchen.oxydition.protedition.common.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertTopicReferenceBean;

/**
 * Dialogfenster für die Operation zum Einfügen von Tagesordnungspunktverweisen.
 * Enthält eine Dropdownmenü zur Wahl des Bandes und mehrere Textfelder zur
 * Eingabe der Tagesordnungs(unterpunkt)nummer.
 * 
 * @author Maximilian Schrott
 *
 */

public class InsertTopicReferenceDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = 1L;

	private JComboBox<String> volumeInput;
	private JTextField protocolNumberInput;
	private JTextField topicNumberInput;
	private JTextField subTopicNumberInput;
	private JTextField subSubTopicNumberInput;
	private JTextField subSubSubTopicNumberInput;
	private InsertTopicReferenceBean inputBean;

	public InsertTopicReferenceDialog(JFrame owner, String title, boolean modal, String[] volumeTitles) {
		super(owner, title);

		volumeInput = addComboBox("Band", volumeTitles);
		addEmptySpace();
		protocolNumberInput = addField("Protokollnummer");
		topicNumberInput = addField("Tagesordnungspunktnummer");
		subTopicNumberInput = addField("Unterpunktsnummer");
		subSubTopicNumberInput = addField("Unterunterpunktsnummer");
		subSubSubTopicNumberInput = addField("Unterunterunterpunktsnummer");

		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	// aktiviert Dialog
	public InsertTopicReferenceBean displayDialog(int protocolVolumeIndex) {
		inputBean = new InsertTopicReferenceBean();
		okButton.addActionListener(new InsertTopicReferenceOkButtonListener(this));

		volumeInput.setSelectedIndex(protocolVolumeIndex);
		readyDialog();
		return inputBean;
	}

	private class InsertTopicReferenceOkButtonListener implements ActionListener {

		private InsertTopicReferenceDialog dlg;

		public InsertTopicReferenceOkButtonListener(InsertTopicReferenceDialog dlg) {
			this.dlg = dlg;
		}

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dlg.dispose();
			}

		}

		// Überprüft, ob alle benötigten Elemente eingegeben wurden (im
		// benötigten
		// Format)
		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			// TODO Ergänze Bandnummer
			if (!isFilled(protocolNumberInput)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die Protokollnummer eingeben; \n";

			} else if (!isValidProtocollNumber(protocolNumberInput)) {
				allValid = false;
				notAllValidMessage += "Wert der Protokollnummer (ein-dreistellige Zahl erwartet) \n";
			}

			if (!isFilled(topicNumberInput)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für die Tagesordnungspunktnummer eingeben; \n";

			} else if (!isValidTopicNumber(topicNumberInput)) {
				allValid = false;
				notAllValidMessage += "Wert der Tagesordnungspunktnummer ungültig (ein- oder zweistellige Dezimalzahl erwartet); \n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}

		}

		// Liest die Eingabefelder aus und speichert den Inhalt in der Bean
		private InsertTopicReferenceBean extractInput() {

			inputBean = new InsertTopicReferenceBean();

			// Für den Band wird nur die Position im Array zurückgegeben, aus
			// der
			// die relevanten Informationen später leicht rekonstruiert werden
			// können
			inputBean.volumeIndex = volumeInput.getSelectedIndex();
			inputBean.protocolNumber = protocolNumberInput.getText();
			inputBean.topicNumber = topicNumberInput.getText();
			inputBean.subTopicNumber = subTopicNumberInput.getText();
			inputBean.subSubTopicNumber = subSubTopicNumberInput.getText();
			inputBean.subSubSubTopicNumber = subSubSubTopicNumberInput.getText();

			inputBean.isFilled = true;

			return inputBean;

		}

		// Überprüft ob in ein Eingabefeld eine Eingabe gemacht wurde
		private boolean isFilled(JTextField inputField) {
			String inputText = inputField.getText();
			return !inputText.isEmpty();
		}

		// Überprüft ob die Protokollnummer eine höchsten dreistellige Zahl ist
		private boolean isValidProtocollNumber(JTextField protocolNumberInput) {
			String protokollNumber = protocolNumberInput.getText();
			return protokollNumber.matches("[0-9]{1,3}");

		}

		// Überprüft ob die Protokollnummer eine höchsten zweistellige Zahl ist
		private boolean isValidTopicNumber(JTextField topicNumberInput) {
			String topicNumber = topicNumberInput.getText();
			return topicNumber.matches("[0-9]{1,2}");
		}

	}
}
