package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertProtocolHeaderBean;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.schemaaware.SchemaAwareHandlerResult;

/**
 * Abstrakte Oberklasse für Operationen, mit denen die Benutzer*innen den
 * Dokumentenkopf in ein Protokoll einfügen können. Diese Klasse regelt den
 * Operationsablauf und das Einfügen der DokumentenID als xml:id-Attribut in das
 * <TEI>-Wurzelelement. Die editionsspezifische Abfrage der Benutzereingaben und
 * das Erzeugen des Dokumentenkopfinhalts in XML muss von erbenden Klassen
 * implementiert werden.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class InsertProtocolHeader extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Nimmt die Eingaben der Benutzer*innen für die Grunddaten des Protokolls entgegen und konstruiert daraus einen Dokumentenkopf";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap arg1)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		boolean noHead = checkIfDocumentAlreadyHasHead(documentController);

		if (!noHead) {
			JOptionPane.showMessageDialog(oxygenFrame,
					"Es kann nicht mehr als ein Dokumentenkopf pro Protokoll eingefügt werden",
					"Operation nicht durchführbar", JOptionPane.WARNING_MESSAGE);
			return;
		}

		InsertProtocolHeaderBean inputBean = getInputData(oxygenFrame);

		if (inputBean.isFilled) {
			SchemaAwareHandlerResult insertResult = insertProtocolHeaderFragment(inputBean);

			if (insertResult != null) {
				ProtocolUtility.moveCursorToInsertLocation(insertResult, authorAccess);
			}
		}

	}

	private SchemaAwareHandlerResult insertProtocolHeaderFragment(InsertProtocolHeaderBean inputBean)
			throws AuthorOperationException {

		String xpathLocation = "//div[@type='Dokumentenkopf']";

		String volumeID = ProtocolUtility.getVolumeID(documentController);
		insertProtocolNumberIntoTEIHeader(inputBean, documentController);
		insertProtocolXMLID(volumeID, inputBean.protNumWithLeadingZeros, documentController);
		String protocolHeaderFragment = constructProtocolHeaderFragment(inputBean);
		SchemaAwareHandlerResult insertResult = documentController.insertXMLFragmentSchemaAware(protocolHeaderFragment,
				xpathLocation, AuthorConstants.POSITION_INSIDE_FIRST);

		return insertResult;
	}

	private void insertProtocolXMLID(String volumeID, String protocolNumberWithLeadingZeros,
			AuthorDocumentController documentController) {

		AuthorElement teiNode = documentController.getAuthorDocumentNode().getRootElement();

		String protID = volumeID + "p" + protocolNumberWithLeadingZeros;
		AttrValue protIDAttr = new AttrValue(protID);

		documentController.setAttribute("xml:id", protIDAttr, teiNode);

	}

	private void insertProtocolNumberIntoTEIHeader(InsertProtocolHeaderBean inputBean,
			AuthorDocumentController documentController) throws AuthorOperationException {

		String titlePath = "//titleStmt/title[@type='Protokoll']";

		String titleProtocolNumber = "p" + inputBean.protNumWithLeadingZeros;
		AttrValue titleProtocolNumberAttr = new AttrValue(titleProtocolNumber);

		AuthorNode[] protocolTitleNodes = documentController.findNodesByXPath(titlePath, false, false, true);
		AuthorElement protocolTitleElement = (AuthorElement) protocolTitleNodes[0];

		documentController.insertXMLFragment(inputBean.protNum, protocolTitleElement,
				AuthorConstants.POSITION_INSIDE_FIRST);

		protocolTitleElement.setAttribute("n", titleProtocolNumberAttr);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return null;
	}

	private boolean checkIfDocumentAlreadyHasHead(AuthorDocumentController documentController)
			throws AuthorOperationException {

		AuthorNode[] numNodes = documentController.findNodesByXPath("//div[@type='Dokumentenkopf']/head/num", false,
				false, true);

		return numNodes.length == 0;
	}

	protected abstract InsertProtocolHeaderBean getInputData(JFrame parentFrame);

	protected abstract String constructProtocolHeaderFragment(InsertProtocolHeaderBean inputBean);

}
