package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.UnpublishedSourceReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen einen Verweis auf einen
 * Eintrag aus der Registerdatenbank für ungedruckte Quellen einfügen können
 * (als <rs>-Element). Der Verweis übernimmt den Bezeichner der Quelle und des
 * Quellenbestandes, dessen Teil sie ist.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertUnpubSourceInfo extends ReadRegisterOperation {

	// Operationsargumente einrichten
	protected static final String ARGUMENT_URL = "URL";

	public static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, "Die Url der Registerdatei, etwa: "
					+ "http://user:passwort@www.example.com:port/exist/webdav/db/register.xml") };

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen eine ungedruckte Quelle aus dem Register "
				+ "auswählen können. Der Bezeichner der Quelle + der Kennname des überbeordneten Bestands oder "
				+ "Archivs wird an der Cursorposition eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		Object urlArgVal = args.getArgumentValue(ARGUMENT_URL);

		if (!OperationUtility.isValidArgument(urlArgVal)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			UnpublishedSourceReadRegister registerReader = new UnpublishedSourceReadRegister((String) urlArgVal);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame,
					"Verweis auf ungedruckte Quelle eingeben", eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertUnpubSourceFragment(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	private void insertUnpubSourceFragment(ListSelectionBean input) throws AuthorOperationException {
		String unpubSourceFragment = constructUnpubSourceFragment(input);
		documentController.insertXMLFragment(unpubSourceFragment, caretPosition);
	}

	protected String constructUnpubSourceFragment(ListSelectionBean input) {

		String selectedID = input.id;
		String selectedSourceName = input.value;
		String displayName = createDisplayName(selectedSourceName);

		String fragment = "<rs xmlns='http://www.tei-c.org/ns/1.0' type='Ungedruckt' key='" + selectedID + "'>";
		fragment += displayName + "</rs>";

		return fragment;
	}

	protected String createDisplayName(String selectedSourceName) {
		// Falls vorhanden, den Archivnamen vor dem ":" entfernen, um den
		// Anzeigenamen aus [Bestand und] Quellennummer zu erzeugen
		String[] selectedSourceNameSplit = selectedSourceName.split(": ");
		String splitSourceName = selectedSourceNameSplit[selectedSourceNameSplit.length - 1];

		return XMLUtility.escapeXML(splitSourceName);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
