package de.historischekommission_muenchen.oxydition.protedition.common.operations;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.operations.TagKeywordOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation, die es den Benutzer*innen erlaubt einen markierten
 * Textausschnitt als Fundstellenabgabe für ein Gesetz, das in der
 * Registerdatenbank für Sachschlagwörter enthalten ist, zu kennzeichnen. Eine
 * spezialisierung der Operation zur Schlagwortmarkierung, die dem eingefügten
 * <rs>-Element ein zusätzlichen subtype-Attribut gibt.
 * 
 * @author Maximilian Schrott
 *
 */
public class TagLawCitation extends TagKeywordOperation {

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen die markierte Textstelle mit einer/m auswählbare/n Verordnung/Gesetz "
				+ " im Sachschlagwortsregister zu verknüpfen und kennzeichnet sie als Fundstelle für den Gesetzestext.";
	}

	@Override
	protected String getFrameName() {
		return "Auswahl als Sachschlagwort markieren";
	}

	@Override
	protected String constructTagFragment(ListSelectionBean input, String typeArgVal) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<rs " + XMLUtility.TEIXMLNS + " type='" + typeArgVal + "' ");
		fragmentBuilder.append("subtype='Gesetzfundstelle' ");
		fragmentBuilder.append("key='" + input.id + "'/>");

		return fragmentBuilder.toString();
	}
}
