package de.historischekommission_muenchen.oxydition.protedition.minrprot.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import de.historischekommission_muenchen.oxydition.protedition.common.beans.InsertProtocolTopicBean;
import de.historischekommission_muenchen.oxydition.protedition.common.dialogs.InsertProtocolTopicDialog;
import de.historischekommission_muenchen.oxydition.common.beans.InsertStructureElementBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertStructureElement;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen den einen
 * Tagesordungsunterpunkt oder -schattenunterpunkt in ein MinRProt-Protokolls
 * einfügen können. Die Operation nimmt Benutzereingaben zu Top-Nummer und
 * Überschrift und Informationen zu editorischen Änderungen entgegen und fügt
 * ein <div>-Element ins Protokoll ein zusammen mit den Titelangaben als <head>.
 * 
 * @author Maximilian Schrott
 * 
 */
public class InsertProtocolSubTopic_MinRProt extends InsertStructureElement {

	@Override
	public String getDescription() {
		return "Die Benutzer*innen geben Titel und Nummer eines neuen Tagesordnungsunterpunktes in einem MinRProt-Protokoll ein, "
				+ "der dann an der vom Cursor markierten Stelle eingesetzt wird.";
	}

	@Override
	protected InsertProtocolTopicBean getInputData(JFrame oxygenFrame, String dialogTitle) {
		InsertProtocolTopicDialog dlg = new InsertProtocolTopicDialog(oxygenFrame,
				InsertProtocolTopicDialog.MINRPROT_SUBTOP);
		InsertProtocolTopicBean input = dlg.displayDialog();

		return input;
	}

	@Override
	protected String constructTopicXMLFragment(InsertStructureElementBean inputBean, String idPrefix) {

		InsertProtocolTopicBean input = (InsertProtocolTopicBean) inputBean;

		String normNum = normaliseNormNum(input.normNum);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<div " + XMLUtility.TEIXMLNS + " ");

		if (input.ishiddenTopic) {
			fragmentBuilder.append("type='Schatten-Unterpunkt' ");
		} else {
			fragmentBuilder.append("type='Unterpunkt' ");
		}

		fragmentBuilder.append("n='" + normNum + "' ");
		fragmentBuilder.append("xml:id='" + idPrefix + "u" + normNum + "'>");
		fragmentBuilder.append(constructHeadXMLFragment(input) + " </div>");

		return fragmentBuilder.toString();

	}

	@Override
	protected String constructHeadXMLFragment(InsertStructureElementBean inputBean) {

		InsertProtocolTopicBean input = (InsertProtocolTopicBean) inputBean;

		StringBuilder headBuilder = new StringBuilder();
		headBuilder.append("<head " + XMLUtility.TEIXMLNS + ">");

		// Je nachdem welche Elemente des Tagespunkts von den Benutzer*innen
		// eingefügt wurden <add>-Elemente in die Überschrift einfügen
		if (input.addedNumber || input.addedAll) {
			headBuilder.append("<add " + XMLUtility.TEIXMLNS + ">");
		}

		headBuilder.append("<num " + XMLUtility.TEIXMLNS + ">" + input.elementNum + "</num>");

		if (input.addedNumber) {
			headBuilder.append("</add>");
		}

		if (input.addedTitle) {
			headBuilder.append("<add " + XMLUtility.TEIXMLNS + ">");
		}

		headBuilder.append(input.title);

		if (input.addedTitle || input.addedAll) {
			headBuilder.append("</add>");
		}

		headBuilder.append("</head>");

		return headBuilder.toString();
	}

	@Override
	protected boolean isCaretPositionValid(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {
		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

		boolean withinTop = XMLUtility.elementTypeEquals(surroundNode, "Top");
		boolean withinSubTop = XMLUtility.elementTypeEquals(surroundNode, "Unterpunkt");
		boolean withinHiddenTop = XMLUtility.elementTypeEquals(surroundNode, "Schatten-Unterpunkt");

		return (withinTop || withinSubTop || withinHiddenTop);
	}

	@Override
	protected String getInvalidCaretPositionMessage() {

		return "An dieser Position darf kein Tagesordnungsunterpunkt eingefügt werden."
				+ "\nFügen Sie einen Tagesordnungsunterpunkt bitte innerhalb eines "
				+ "Tagesordnungspunkt/Unterpunkt/Schattenunterpunkt-Abschnitts ein,"
				+ "\naber nicht innerhalb eines anderen Unterabschnitts";
	}

	@Override
	protected String findIdPrefix(int caretPosition, AuthorDocumentController documentController)
			throws BadLocationException {
		return XMLUtility.getSurroundNodeID(caretPosition, documentController);
	}

}
