package de.historischekommission_muenchen.oxydition.protedition.weimprot.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import de.historischekommission_muenchen.oxydition.protedition.weimprot.beans.InsertSourceDocumentHeaderBean;

/**
 * Dialogfenster für die Operation zum Einfügen des Dokumentenkopfs in
 * Quelldokumente der Edition WeiMProt. Enthält Textfelder für die
 * Dokumentennummer und den Dokumententitel.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertSourceDocumentHeaderDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = -6159599103072835791L;

	private JTextField numField;
	private JTextField titleField;

	public InsertSourceDocumentHeaderBean input;

	public InsertSourceDocumentHeaderDialog(JFrame parentFrame) {
		super(parentFrame, "Quelldokumentenkopf einfügen");

		numField = addField("Dokumentennummer");
		titleField = addField("Dokumententitel");

		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
	}

	public InsertSourceDocumentHeaderBean displayDialog() {

		input = new InsertSourceDocumentHeaderBean();

		okButton.addActionListener(new OkButtonListener());
		readyDialog();

		return input;
	}

	private class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}
		}

		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextfield(numField)) {

				allValid = false;
				notAllValidMessage += "Kein Wert für Dokumentennummer eingegeben; \n";
			} else if (!ProtocolUtility.isValidProtocolNumberInput(numField)) {

				allValid = false;
				notAllValidMessage += "Die eingegebene Dokumentennummer hat keine gültige Form (ein- bis dreistellige Zahl); \n";
			}

			if (!DialogUtility.isFilledTextfield(titleField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Dokumententitel eingegeben;";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}
		}

		private void extractInput() {

			input.sourceDocNum = numField.getText();
			input.sourceDocNumWithLeadingZeros = ("000" + numField.getText()).substring(numField.getText().length());

			input.sourceDocTitle = titleField.getText();
			input.isFilled = true;
		}
	}
}
