package de.historischekommission_muenchen.oxydition.protedition.weimprot.operations;

import java.util.regex.Pattern;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionTextfieldBean;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.InsertFromRegisterWithTextfield;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Verweis (als
 * <bibl>-Element) auf ein Amts- oder Gesetztblatt einfügen können. Der Verweis
 * ist mit der entsprechenden Registerdatenbank verknüpft und übernimmt den dort
 * hinterlegten Titel. Außerdem hat die Benutzer*innen die Möglichkeit die
 * genauen Textstelle (z.B. Seitenzahl) einzugeben, die dann zur Literaturangabe
 * hinzugefügt wird.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertLawGazetteInfo extends InsertFromRegisterWithTextfield {

	@Override
	public String getDescription() {

		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen Zeitungs-/Zeitschrifteneintrag aus dem Register auswählen und den genauen Fundort eingeben können."
				+ "Der Titel des Peridikums wird zusammen mit der Fundstelle an der Cursorposition eingetragen.";
	}

	@Override
	protected String constructEntityFragment(ListSelectionTextfieldBean input) {
		String selectedValue = input.id;
		String[] selectedSplit = selectedValue.split(Pattern.quote("$#"));

		String id = selectedSplit[0];

		// Das letzte Element des ValueStrings als Titel benutzen, entweder
		// Haupttitel oder falls vorhanden Kürzel
		String title = selectedSplit[selectedSplit.length - 1];
		title = XMLUtility.escapeXML(title);
		String scope = XMLUtility.escapeXML(input.textInput);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<bibl " + XMLUtility.TEIXMLNS + " type='Gesetzesblatt' ");
		fragmentBuilder.append("corresp='" + id + "'>");
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + ">" + title + "</title>");
		if (!scope.isEmpty()) {
			fragmentBuilder.append(" <biblScope>" + scope + "</biblScope>");
		}
		fragmentBuilder.append("</bibl>");

		return fragmentBuilder.toString();
	}

	@Override
	protected void initializeVariables() {
		nodeVal = "//*[local-name()='listBibl']/*[local-name()='bibl']";
		expressionVal = "//*[local-name()='title' and @type='Name']"
				+ " + \" \" + //*[local-name()='title' and @type='Zusatz']";
		idVal = "@xml:id + \"$#\" + " + "//*[local-name()='title' and @type='Name'] + \"$#\" + "
				+ "//*[local-name()='title' and @type='Kürzel']";

		dialogTitle = "Verweis auf Gesetzes-/Amtsblatt einfügen";
		textFieldLabel = "Genaue Textstelle, z.B. Seitenzahl";
	}
}
