package de.historischekommission_muenchen.oxydition.protedition.weimprot.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.UnpublishedSourceReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.operations.InsertUnpubSourceInfo;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxydition-Author-Operation mit der die Benutzer*innen einen Verweis auf einen
 * Eintrag aus der Registerdatenbank für ungedruckte Quellen in den
 * Metadatenappart eine Dokuments einfügen können. Dadurch geben sie an, dass die
 * referenzierte Quelle die/ eine der Grundlage(n) das aktuellen Dokuments ist.
 * Die Referenz ist ein <bibl>-Element, dass in das <sourceDesc>-Element des
 * <teiHeader>s eingefügt wird.
 * 
 * @author Maximilian Schrott
 */
public class InsertMetadataProtocolSource extends InsertUnpubSourceInfo {

	@Override
	public void doOperation(final AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		Object urlArgVal = args.getArgumentValue(ARGUMENT_URL);

		if (!OperationUtility.isValidArgument(urlArgVal)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			UnpublishedSourceReadRegister registerReader = new UnpublishedSourceReadRegister((String) urlArgVal);

			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame,
					"Metadaten für Dokumentenquelle einfügen", eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				String unpubSourceFragment = constructUnpubSourceFragment(input);
				insertUnpubSourceFragment(unpubSourceFragment, documentController);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	@Override
	protected String constructUnpubSourceFragment(ListSelectionBean input) {

		String selectedID = input.id;
		String selectedSourceName = input.value;

		String displayName = createDisplayName(selectedSourceName);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<bibl " + XMLUtility.TEIXMLNS + " type='Quellennachweis'");
		fragmentBuilder.append(" corresp='" + selectedID + "'>");
		fragmentBuilder.append(displayName + "</bibl>");

		return fragmentBuilder.toString();
	}

	private void insertUnpubSourceFragment(String UnpubSourceFragment, AuthorDocumentController documentController)
			throws AuthorOperationException {

		String xPathLocation = "//fileDesc/sourceDesc";
		documentController.insertXMLFragmentSchemaAware(UnpubSourceFragment, xPathLocation,
				AuthorConstants.POSITION_INSIDE_LAST);

	}

}
