package de.historischekommission_muenchen.oxydition.protedition.weimprot.operations;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.protedition.common.utilities.ProtocolUtility;
import de.historischekommission_muenchen.oxydition.protedition.weimprot.dialogs.InsertSourceDocumentReferenceDialog;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Verweis auf ein
 * Quelldokument einfügen können. Der Verweis (ein <rs>-Element) wird aus der
 * Eingabe der Dokumentennummer durch die Benutzer*innen rekonstruiert.
 * 
 * @author Maximilian Schrott
 *
 */
public class InsertSourceDocumentReference extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Fügt eine Referenz auf eine Quelldokument im selben Band ein";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		String protocolVolumeId = ProtocolUtility.getVolumeID(documentController);

		InsertSourceDocumentReferenceDialog dlg = new InsertSourceDocumentReferenceDialog(oxygenFrame);

		String documentNumber = dlg.displayDialog();

		if (!documentNumber.isEmpty()) {
			String sourceDocumentReferenceFragment = constructSourceDocumentReferenceFragment(documentNumber,
					protocolVolumeId);
			documentController.insertXMLFragment(sourceDocumentReferenceFragment, caretPosition);
		}

	}

	private String constructSourceDocumentReferenceFragment(String documentNumber, String protocolVolumeId) {

		String sourceDocumentId = constructSourceDocumentId(documentNumber, protocolVolumeId);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<rs " + XMLUtility.TEIXMLNS + " type='Quelldokumentreferenz' ");
		fragmentBuilder.append("key='" + sourceDocumentId + "'>");
		fragmentBuilder.append("Dok. " + documentNumber + "</rs>");

		return fragmentBuilder.toString();

	}

	private String constructSourceDocumentId(String documentNumber, String protocolVolumeId) {

		String documentNumberNormalised = ProtocolUtility.prefixZerosForLength(documentNumber, 3);
		String sourceDocumentID = protocolVolumeId + "d" + documentNumberNormalised;

		return sourceDocumentID;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
