package de.historischekommission_muenchen.oxydition.register.dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.ModularGridBagDialog;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.register.beans.GeoPointCoordinatesBean;

/**
 * Dialogfenster für Operationen bei denen die Benutzer*innen
 * Geopunktkoordinaten einfügen oder bearbeiteen. Enthält ein Textfeld für den
 * Längen- und eines für den Breitengrad, die die jeweilige Eingabe auf
 * Gültigkeit überprüfen sowie ein Panel mit Erläuterungstext.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterGeoPointCoordinatesDialog extends ModularGridBagDialog {

	private static final long serialVersionUID = -4429067796819785365L;

	private JTextField latField;
	private JTextField lonField;

	private GeoPointCoordinatesBean input;

	private String helpText = "Geokoordinaten als Dezimalzahlen (mit Punkt) gemäß des Koordinatensystems EPSG:3857[Spherical Mercator] (wie von OpenStreetMap und GoogleMaps verwendet) eintragen.\n"
			+ "Zulässige Werte sind -90.0 bis 90.0 für den Breitengrad und -180.0 bis 180.0 für Längengrade.\n"
			+ "Beispiel: Der Münchner Marienplatz hätte die Koordinaten Breitengrad: 48.137247, Längengrad: 11.575539";

	public RegisterGeoPointCoordinatesDialog(JFrame parent, String title) {

		super(parent, title);

		latField = addField("Breitengrad");
		lonField = addField("Längengrad");
		addEmptySpace();
		addFullWithHelpTextArea(helpText);
		insertOperationEndButtonPanel("Einfügen", "Abbrechen");
		okButton.addActionListener(new GeoPointCoordinatesOkButtonListener());

	}

	public GeoPointCoordinatesBean displayDialog() {

		input = new GeoPointCoordinatesBean();
		readyDialog();
		return input;
	}

	private class GeoPointCoordinatesOkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			boolean inputValid = validateInput();

			if (inputValid) {

				input = extractInput();
				dispose();
			}
		}

		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isValidLatitude(latField)) {
				allValid = false;
				notAllValidMessage += "Keine gültige Eingabe für den Breitengrad;\n";
			}

			if (!DialogUtility.isValidLongitude(lonField)) {
				allValid = false;
				notAllValidMessage += "Keine gültige Eingabe für den Längengrad;\n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}
		}

		private GeoPointCoordinatesBean extractInput() {

			input.latitude = latField.getText();
			input.longitude = lonField.getText();

			input.isFilled = true;

			return input;
		}

	}

}
