package de.historischekommission_muenchen.oxydition.register.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.text.Document;

import de.historischekommission_muenchen.oxydition.common.dialogs.OxyditionOperationDialog;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.OxyditionInputFilter;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.SelectionListSearchFieldListener;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.UpdatingJList;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.register.beans.RegisterKeywordBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.elements.ReferencedEntryButton;

/**
 * Dialogfenster zum Einfügen oder Bearbeiten von Sachschlagworten in der
 * Registerdatenbank. Das Fenster ist in zwei Hälften geteilt. Auf der rechten
 * Seite können die Benutzer*innen andere Sachschlagworte aus einer Liste
 * auswählen und als Querverweise markieren. Auf der linken Seite wird das
 * Sachschlagwort selbst eingetragen bzw. bearbeitet und eine Liste der
 * markierten Querverweise angezeigt. Die Querverweise können mittels der
 * "x"-Button neben ihnen wieder gelöscht werden.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterKeywordNestedDialog extends OxyditionOperationDialog {

	private static final long serialVersionUID = 4329393135289306695L;

	private String[] eintraege;
	private String[] ids;

	private JPanel newEntryPanel;
	private JPanel registerPanel;

	private JPanel crossReferencePanel;
	private JPanel registerSearchPanel;

	private GridBagConstraints crossReferencePanelConstraints;

	private JTextField termField;
	private JTextField registerSearchField;
	private UpdatingJList<String> registerList;
	private JButton crossReferenceButton;

	private RegisterKeywordBean input;

	private final float leftAlign = Component.LEFT_ALIGNMENT;

	// Faktisch unbegrenzte Breite für maximumSize-Anweisungen (nötig, da die
	// Höhe der Elemente begrenz werden soll)
	private final int MAXSIZEH = 50000;

	private final int H_SIZE = 700;
	private static int V_SIZE = 600;

	// Standartelementgrößen
	private final Dimension LABELMAXSIZE = new Dimension(MAXSIZEH, 20);
	private final Dimension TEXTFIELDMAXSIZE = new Dimension(MAXSIZEH, 20);

	public RegisterKeywordNestedDialog(JFrame oxygenFrame, String title, String[] eintraege, String[] ids) {
		super(oxygenFrame, title);

		this.eintraege = eintraege;
		this.ids = ids;
		assembleMainContent();

		okButton.addActionListener(new OkButtonListener());
	}

	/**
	 * Zeigt das Dialogfenster ab und gibt ein Datenhaltungsobjekt zurück, nachdem
	 * es gschlossen wurde
	 */
	public RegisterKeywordBean displayDialog() {

		input = new RegisterKeywordBean();

		readyDialog();

		return input;
	}

	// Alternativer Aufruf für Bearbeitung
	public RegisterKeywordBean displayDialog(RegisterKeywordBean keywordBean) {
		this.input = new RegisterKeywordBean(keywordBean);

		fillInKeywordData();
		readyDialog();

		return input;

	}

	private void fillInKeywordData() {
		termField.setText(input.keywordTerm);

		for (int i = 0; i < input.getKeywordCrossReferenceSize(); i++) {

			String crossReferenceTerm = input.getCrossReferenceTerm(i);
			String crossReferenceId = input.getCrossReferenceId(i);
			addReferenceKeywordLabel(crossReferenceTerm, crossReferenceId);
		}
	}

	private void assembleMainContent() {

		insertNewEntryPanel();
		insertRegisterPanel();
		insertOperationEndButtonPanel("Hinzufügen", "Abbrechen");
	}

	/* Linke Fensterseite */
	private void insertNewEntryPanel() {

		newEntryPanel = new JPanel();
		newEntryPanel.setLayout(new BoxLayout(newEntryPanel, BoxLayout.Y_AXIS));
		insertTermField();
		newEntryPanel.add(DialogUtility.BIGRIGID);
		insertCrossReferencePanel();

		mainContent.add(newEntryPanel, BorderLayout.CENTER);
	}

	private void insertTermField() {
		JLabel termLabel = createLabel("Schlagwort");
		newEntryPanel.add(termLabel);

		newEntryPanel.add(DialogUtility.SMALLRIGID);

		termField = createTextField();
		newEntryPanel.add(termField);
	}

	private void insertCrossReferencePanel() {

		crossReferencePanel = new JPanel(new GridBagLayout());
		crossReferencePanel.setAlignmentX(leftAlign);
		crossReferencePanel.setBorder(new EmptyBorder(3, 3, 3, 3));

		crossReferencePanelConstraints = setupCrossReferencePanelConstraints();

		// Dummy-Label, damit das Layout von oben her die Elemente einfügt
		crossReferencePanelConstraints.gridy = 1000;
		crossReferencePanelConstraints.weighty = 1;
		JLabel dummyLabel = new JLabel();
		crossReferencePanelConstraints.weightx = 1;
		crossReferencePanel.add(dummyLabel, crossReferencePanelConstraints);

		crossReferencePanelConstraints.gridy = 1;
		crossReferencePanelConstraints.weighty = 0;

		newEntryPanel.add(crossReferencePanel, BorderLayout.CENTER);
	}

	private GridBagConstraints setupCrossReferencePanelConstraints() {
		GridBagConstraints cstr = new GridBagConstraints();
		cstr.gridx = 0;
		cstr.gridy = -1;
		cstr.fill = GridBagConstraints.HORIZONTAL;
		cstr.insets = new Insets(2, 3, 2, 0);
		cstr.anchor = GridBagConstraints.FIRST_LINE_START;

		return cstr;
	}

	/* Rechte Fensterseite */
	private void insertRegisterPanel() {
		registerPanel = new JPanel(new BorderLayout(0, 5));

		insertRegisterPanelTitle();
		insertRegisterSearchPanel();
		insertRegisterButtonPanel();

		mainContent.add(registerPanel, BorderLayout.LINE_END);
	}

	private void insertRegisterPanelTitle() {
		JLabel registerTitleLabel = createLabel("Andere Sachschlagwörter referenzieren");
		registerPanel.add(registerTitleLabel, BorderLayout.PAGE_START);
	}

	private void insertRegisterSearchPanel() {
		registerSearchPanel = new JPanel();
		registerSearchPanel.setLayout(new BoxLayout(registerSearchPanel, BoxLayout.Y_AXIS));
		registerSearchPanel.setPreferredSize(new Dimension(380, 0));

		insertRegisterSearchField();
		registerSearchPanel.add(DialogUtility.BIGRIGID);
		insertRegisterList();

		registerPanel.add(registerSearchPanel, BorderLayout.CENTER);
	}

	private void insertRegisterSearchField() {
		JLabel registerSearchLabel = createLabel("Suche:");
		registerSearchPanel.add(registerSearchLabel);

		registerSearchPanel.add(DialogUtility.SMALLRIGID);

		registerSearchField = createTextField();
		registerSearchPanel.add(registerSearchField);
	}

	private void insertRegisterList() {
		JLabel registerListLabel = createLabel("Referenzierbare Schlagwörter");
		registerSearchPanel.add(registerListLabel);

		registerSearchPanel.add(DialogUtility.SMALLRIGID);

		registerList = new UpdatingJList<String>(eintraege, ids);
		registerList.setSelectedIndex(0);
		Document registerSearchDocument = registerSearchField.getDocument();
		registerSearchDocument.addDocumentListener(new SelectionListSearchFieldListener(registerList));

		JScrollPane registerScroll = new JScrollPane();
		registerScroll.setViewportView(registerList);

		registerScroll.setMaximumSize(new Dimension(MAXSIZEH, 3000));
		registerScroll.setAlignmentX(leftAlign);

		registerSearchPanel.add(registerScroll);
	}

	private void insertRegisterButtonPanel() {
		JPanel registerButtonPanel = new JPanel();

		crossReferenceButton = new JButton("Querverweis");
		crossReferenceButton.addActionListener(new CrossReferenceButtonListener());
		registerButtonPanel.add(crossReferenceButton);

		registerPanel.add(registerButtonPanel, BorderLayout.PAGE_END);
	}

	private JTextField createTextField() {
		JTextField textField = new JTextField();
		textField.setMaximumSize(TEXTFIELDMAXSIZE);
		textField.setAlignmentX(leftAlign);
		OxyditionInputFilter.setForInput(textField);

		return textField;
	}

	private JLabel createLabel(String labelText) {
		JLabel label = new JLabel(labelText);
		label.setMaximumSize(LABELMAXSIZE);
		label.setAlignmentX(leftAlign);

		return label;
	}

	private void readyDialog() {
		this.getContentPane().add(mainContent);
		this.pack();
		this.setSize(H_SIZE, V_SIZE);
		this.setLocationRelativeTo(this.getOwner());
		this.setVisible(true);
	}

	private class OkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent arg0) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}
		}

		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextfield(termField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für Neues Schlagwort eingegeben; \n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}
		}

		private void extractInput() {

			input.keywordTerm = termField.getText();
			input.isFilled = true;
		}
	}

	private class CrossReferenceButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String selectedTerm = registerList.getSelectedValue();

			if (!input.crossReferenceAlreadySet(selectedTerm)) {

				int selectedIndex = registerList.getSelectedIndex();
				String selectedId = registerList.getMetaEntryAt(selectedIndex);

				addReferenceKeywordLabel(selectedTerm, selectedId);
				input.addKeywordCrossReference(selectedTerm, selectedId);
			}
		}
	}

	private void addReferenceKeywordLabel(String term, String id) {

		JLabel referenceKeywordLabel = new JLabel("Querverweis: " + term);

		crossReferencePanelConstraints.gridx = 0;
		crossReferencePanelConstraints.weightx = 1;
		crossReferencePanelConstraints.gridy += 1;

		crossReferencePanel.add(referenceKeywordLabel, crossReferencePanelConstraints);

		crossReferencePanelConstraints.gridx = 1;
		crossReferencePanelConstraints.weightx = 0;

		JButton deleteCrossReferenceButton = new DeleteReferenceKeywordButton(referenceKeywordLabel,
				crossReferencePanel, term);

		crossReferencePanel.add(deleteCrossReferenceButton, crossReferencePanelConstraints);

		crossReferencePanel.revalidate();
		crossReferencePanel.repaint();
	}

	@Override
	protected void cancelButtonAction() {
		dispose();

	}

	private class DeleteReferenceKeywordButton extends ReferencedEntryButton {

		private static final long serialVersionUID = -7975782721252373791L;

		public DeleteReferenceKeywordButton(JLabel label, JPanel container, String entry) {
			super(label, container, entry);
		}

		@Override
		public void deleteKeyword() {

			input.removeKeywordCrossReference(entry);

			container.remove(label);
			container.remove(this);

			container.revalidate();
			container.repaint();
		}
	}
}
