package de.historischekommission_muenchen.oxydition.register.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;

import de.historischekommission_muenchen.oxydition.common.dialogs.OxyditionOperationDialog;
import de.historischekommission_muenchen.oxydition.common.dialogs.elements.OxyditionInputFilter;
import de.historischekommission_muenchen.oxydition.common.utilities.DialogUtility;
import de.historischekommission_muenchen.oxydition.register.beans.LiteratureTitleContentBean;
import de.historischekommission_muenchen.oxydition.register.beans.LiteratureTitlePartBean;
import de.historischekommission_muenchen.oxydition.register.beans.NewLiteraturTitleBean;

/**
 * Dialogfenster für die Operation zum Einfügen neuer Literaturtitel in die
 * Literaturregisterdatenbank.
 * 
 * @author Maximilian Schrott
 *
 */

public class RegisterLiteratureTitleDialog extends OxyditionOperationDialog {

	private static final long serialVersionUID = -6625662352729972745L;

	private static final int MAXSIZEH = 50000;

	private static final Dimension LABELMAXSIZE = new Dimension(MAXSIZEH, 20);
	private static final Dimension TEXTFIELDMAXSIZE = new Dimension(MAXSIZEH, 20);

	private static final int H_SIZE = 1200;

	private static final int V_SIZE = 500;

	private float leftAlign;

	private JPanel inputPanel;
	private JPanel helpPanel;

	private JTextArea fullTitleArea;
	private JTextArea shortTitleArea;
	private JTextField dateField;

	private JTextArea helpTextArea;

	private NewLiteraturTitleBean inputBean;

	public Object inputTextSplit;

	public RegisterLiteratureTitleDialog(JFrame owner, String title) {
		super(owner, title);

		leftAlign = Component.LEFT_ALIGNMENT;

		inputPanel = constructInputPanel();
		mainContent.add(inputPanel, BorderLayout.CENTER);

		helpPanel = constructHelpPanel();
		mainContent.add(helpPanel, BorderLayout.LINE_END);

		insertOperationEndButtonPanel("Übernehmen", "Abbrechen");

		okButton.addActionListener(new NewLiteratureTitleOkButtonListener());

		inputBean = new NewLiteraturTitleBean();
	}

	// Stellt das Panel mit den Eingabefeldern zusammen
	private JPanel constructInputPanel() {

		JPanel inputPanel = new JPanel();
		inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));

		inputPanel.add(createLabel("Volltitel"));

		fullTitleArea = createTextArea(8, 50);
		fullTitleArea.setText("<>.");
		fullTitleArea.setCaretPosition(1);
		fullTitleArea.addFocusListener(new FullTitleFocusListener());
		OxyditionInputFilter.setForInput(fullTitleArea);
		inputPanel.add(fullTitleArea);

		inputPanel.add(createLabel("Kurztitel"));
		shortTitleArea = createTextArea(3, 50);
		OxyditionInputFilter.setForInput(shortTitleArea);
		inputPanel.add(shortTitleArea);

		inputPanel.add(createLabel("Datum"));
		dateField = new JTextField();
		dateField.setMaximumSize(TEXTFIELDMAXSIZE);
		dateField.setAlignmentX(leftAlign);
		inputPanel.add(dateField);

		return inputPanel;
	}

	// Stellt das Panel mitn dem hilfetext zusammen
	private JPanel constructHelpPanel() {

		JPanel helpPanel = new JPanel();
		helpPanel.setLayout(new BoxLayout(helpPanel, BoxLayout.Y_AXIS));

		helpPanel.add(createLabel("Benutzungshinweise:"));
		helpTextArea = createTextArea(14, 30);
		helpTextArea.setEditable(false);

		helpTextArea.setText("Hervorzuhebenden Text in '<>'-Klammern setzten.\n\n"
				+ "Im Volltitel dürfen beliebig viele Abschnitte hervorgehoben werden.\n\n"
				+ "Im Kurztitel darf es nur einen hervorgehobenen Abschnitt geben. \n\n"
				+ "Hervorzuhebende Abschnitte sollten so eng wie möglich eingefasst werden. \n\n"
				+ "'<' oder '>' die im Titel selbst vorkommen, müssen mit einem führenden '\\' freigestellt werden.");
		helpPanel.add(helpTextArea);

		return helpPanel;
	}

	private JLabel createLabel(String labelText) {

		JLabel label = new JLabel(labelText);
		label.setMaximumSize(LABELMAXSIZE);
		label.setAlignmentX(leftAlign);

		return label;

	}

	private JTextArea createTextArea(int rows, int columns) {

		JTextArea textArea = new JTextArea(rows, columns);
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);

		Border textAreaBorder = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
		textArea.setBorder(textAreaBorder);

		return textArea;
	}

	// Splittet einen Text entlang der Zeichen für
	private String[] splitTextAtHiglights(String inputText) {
		// Vier \ da sowohl der escapende als auch der escapte \ im
		// String escapet werden müssen
		return inputText.split("(?<!\\\\)[<>]");
	}

	/**
	 * Zeigt das Dialogfenster an und nimmt Benutzereingaben entgegen
	 * 
	 * @return das Datenhaltungsobjekt mit den Benutzereingaben
	 */
	public NewLiteraturTitleBean displayDialog() {
		readyDialog();
		return inputBean;
	}

	private void readyDialog() {

		getContentPane().add(mainContent);
		pack();
		setSize(H_SIZE, V_SIZE);
		setLocationRelativeTo(getOwner());
		setVisible(true);
	}

	/**
	 * Besetzt die Eingabefelder des Dialogfensters mit den übergebenen Inhalten vor
	 * 
	 * @param content - Datenhaltzungsobjekt mit dem vorhandenen Inhalt eines
	 *                RDB-Literatureintrags
	 */
	public void presetContent(LiteratureTitleContentBean content) {

		fullTitleArea.setText(content.fullTitle);
		shortTitleArea.setText(content.shortTitle);
		dateField.setText(content.date);

	}

	// Verhalten für den "Einfügen"-Button (Überprüft Eingaben und überführt sie
	// in Datumshaltungsobjekt)
	private class NewLiteratureTitleOkButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {

			boolean inputValid = validateInput();

			if (inputValid) {
				extractInput();
				dispose();
			}

		}

		// Überprüft ob Eingaben vollständig und gültig
		private boolean validateInput() {

			boolean allValid = true;
			String notAllValidMessage = "";

			if (!DialogUtility.isFilledTextArea(fullTitleArea)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für den Volltitel eingegeben; \n";
			} else if (!validateFullTitleArea()) {
				allValid = false;
				notAllValidMessage += "Volltitel ungültig. Er muss eine oder mehrere hervorgehobene Passagen (in '<>', '<' darf nicht direkt auf '>' folgen) sowie weiteren Text enthalten und mit einem Punkt '.' abgeschlossen werden;\n";
			}

			if (!DialogUtility.isFilledTextArea(shortTitleArea)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für den Kurztitel eingegeben; \n";
			} else if (!validateShortTitleArea()) {
				allValid = false;
				notAllValidMessage += "Kurztitel ungültig. Er darf nicht leer sein und darf höchstens eine hervorgehobene Passage (in '<>') enthalten.\n";
			}

			if (!DialogUtility.isFilledTextfield(dateField)) {
				allValid = false;
				notAllValidMessage += "Kein Wert für das Datum eingegeben; \n";
			}

			if (allValid) {
				return true;
			} else {
				displayInputNotAllValidMessage(notAllValidMessage);
				return false;
			}

		}

		private boolean validateShortTitleArea() {
			String inputText = shortTitleArea.getText();
			String hideEscapeInputText = hideEscapedCharacters(inputText);

			// Darf entweder genau einen oder keinen durch '<>' hervorgehobenen Ausschnitt
			// haben
			return hideEscapeInputText.matches("((<[^<>]+?>)([^<>]+|$))|([^<>]+)");
		}

		private boolean validateFullTitleArea() {

			String inputText = fullTitleArea.getText();
			String escapedInputText = hideEscapedCharacters(inputText);

			// Darf entweder keinen hervorgehobenen Text (umgeben von '<>') enthalten oder
			// abwechselnd hervorgehobenen und nicht hervorgehobenen Text, wobei keine zwei
			// hervorgehobenen Textabschnitte direkt aufeinander folgen dürfen, und der
			// Titel insgesamt auf einem nichthervorgehobenen ABschnitt enden muss.
			return escapedInputText.matches("((<[^<>]+?>)?[^<>]+)+");
		}

		private String hideEscapedCharacters(String inputText) {

			String hideEscapeText = inputText.replace("\\<", "");
			hideEscapeText = hideEscapeText.replace("\\>", "");

			return hideEscapeText;
		}

		private void extractInput() {

			String fullTitleText = fullTitleArea.getText();
			inputBean.fullTitle = divideTitleIntoParts(fullTitleText);

			String shortTitleText = shortTitleArea.getText();
			inputBean.shortTitle = divideTitleIntoParts(shortTitleText);

			inputBean.date = dateField.getText();
			inputBean.isFilled = true;

		}

		private LiteratureTitlePartBean[] divideTitleIntoParts(String titleText) {

			boolean startsWithHiglight = titleText.startsWith("<");
			String[] titleTextParts = splitTextAtHiglights(titleText);

			// Beginnt der Titel mit einem hervorgehobenen Abschnitt, ist das erste Element
			// in titleTextParts ein leerer String und muss entfernt werden
			if (startsWithHiglight) {
				titleTextParts = Arrays.copyOfRange(titleTextParts, 1, titleTextParts.length);
			}

			LiteratureTitlePartBean[] titlePartsBean = new LiteratureTitlePartBean[titleTextParts.length];

			// Wenn der Titel mit einem hervorgehobenen Abschnitt beginnt, sind alle
			// Abschnitt mit gerader Nummer hervorgehoben, sonst alle mit ungerader Nummer
			for (int i = 0; i < titleTextParts.length; i++) {
				String titleTextPart = titleTextParts[i];

				if (i % 2 == 0 && startsWithHiglight) {
					titlePartsBean[i] = new LiteratureTitlePartBean(titleTextPart, true);
				} else if (i % 2 == 1 && !startsWithHiglight) {
					titlePartsBean[i] = new LiteratureTitlePartBean(titleTextPart, true);
				} else {
					titlePartsBean[i] = new LiteratureTitlePartBean(titleTextPart, false);
				}
			}
			return titlePartsBean;
		}

	};

	// Verhalten für das Volltext-Eingabefeld bei Verlust des Focus (konsturiert
	// Kurztitelvorschlag und Datumsvorschlag)
	private class FullTitleFocusListener implements FocusListener {

		@Override
		public void focusGained(FocusEvent e) {

		}

		@Override
		public void focusLost(FocusEvent e) {

			String fullTitleText = fullTitleArea.getText();
			String shortTitleText = shortTitleArea.getText();
			String dateText = dateField.getText();

			if (shortTitleText.isEmpty()) {
				String highlightedText = getHighlightedText(fullTitleText);
				shortTitleArea.setText(highlightedText);
			}

			if (dateText.isEmpty()) {
				String extractedDate = extractDateSubstring(fullTitleText);
				dateField.setText(extractedDate);
			}
		}

		private String getHighlightedText(String text) {

			String[] fullTitleSplit = splitTextAtHiglights(text);
			StringBuilder highlightedTextBuilder = new StringBuilder("<");

			for (int i = 1; i < fullTitleSplit.length; i += 2) {
				if (i > 1) {
					highlightedTextBuilder.append("/");
				}
				highlightedTextBuilder.append(fullTitleSplit[i]);
			}

			highlightedTextBuilder.append(">");
			return highlightedTextBuilder.toString();
		}

		private String extractDateSubstring(String text) {
			// Regex sucht nach mind. 3stelligen Zahlen auf die optional ")"
			// folgt und danach entweder "." und das Stringende oder ", S."
			Pattern datePattern = Pattern.compile("[\\d]{3,}(?=[)]?\\.$|[)]?, S\\.)");
			Matcher dateMatcher = datePattern.matcher(text);

			if (dateMatcher.find()) {
				return dateMatcher.group(0);
			} else {
				return "";
			}
		}
	}

	@Override
	protected void cancelButtonAction() {
		dispose();

	}
}
