package de.historischekommission_muenchen.oxydition.register.dialogs.elements;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Abstrakte JButton-Spezialisierung, die das Grundmuster bietet für einen
 * button, mit dem ein, in einem Dialogfenster angezeigter, Querverweis wieder
 * entfernt werden kann.
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class ReferencedEntryButton extends JButton {

	private static final long serialVersionUID = -1751308196945816262L;
	protected JLabel label;
	protected JPanel container;
	protected String entry;
	protected int type;

	/**
	 * Konstruktor
	 * 
	 * @param label
	 *            - das mit dem Button verknüpfte Label, das den referenzierten
	 *            Eintrag angibt
	 * @param container
	 *            - das JPanel in dem sich der Button und das verknüpfte Label
	 *            befinden
	 * @param entry
	 *            - der Textwert des referenzierten Registereintrags
	 */
	public ReferencedEntryButton(JLabel label, JPanel container, String entry) {

		super("x");

		this.label = label;
		this.container = container;
		this.entry = entry;

		this.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				deleteKeyword();

			}
		});

	}

	/**
	 * Abstrakte Methode, die dazu dient, den Button und das mit ihm verknüpfte
	 * Label zu entfernen, sowie das entsprechende Schlagwort aus der Liste der für
	 * die Referenzierung vorgemerkten Schlagwörter zu löschen
	 */
	public abstract void deleteKeyword();
}
