package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.IOException;

import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.ListSelectionBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SimpleListSelectionDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Abstrakte Oberklasse für Operation mit denen die Benutzer*innen einen
 * RDB-Eintrag mit einem Eintrag in einer anderen RDB-Datei verknüpfen können.
 * Die xml:id des referenzierten Eintrags wird hierfür als corresp-Attribut im
 * Ausgangseintrag eingefügt (falls schon vorhanden wird das Attribut
 * überschrieben).
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class RegisterAddCrossRegisterReference extends ReadRegisterOperation {

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor("URL", ArgumentDescriptor.TYPE_STRING, urlArgDescription) };

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateUrlArgument(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			AuthorElement entryNode = findEntryNode(surroundNode);

			if (entryNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			ReadRegisterBasic registerEntries = getRegisterEntries(args);

			String[] eintraege = registerEntries.getEintrag();
			String[] ids = registerEntries.getID();

			SimpleListSelectionDialog dlg = new SimpleListSelectionDialog(oxygenFrame, getFrameTitle(), eintraege, ids);
			ListSelectionBean input = dlg.displayDialog();

			if (input.isFilled) {
				setRefAttribute(input, entryNode);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	protected void setRefAttribute(ListSelectionBean input, AuthorElement entryNode) {
		AttrValue refValue = new AttrValue(input.id);
		documentController.setAttribute("corresp", refValue, entryNode);
	}

	protected abstract String getFrameTitle();

	protected abstract AuthorElement findEntryNode(AuthorElement surroundNode);

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
