package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen Schlagworteintrag
 * aus der RDB löschen können. Die Operation durchläuft auch die dem gelöschten
 * Eintrag untergeordneten Einträge und löscht auch die Querverweise auf diese.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterDeleteKeyword extends RegisterDeleteEntry {

	@Override
	public String getDescription() {
		return "Löscht, nachdem die Benutzer*innen dies bestätigt haben, dass Schlagwort-<item>-Element, in dem sich der Textcursor befindet, sowie alle Referenzen auf diesen Eintrag.";
	}

	@Override
	protected String constructDeleteMessage(String entryName) {

		return "Wollen Sie den Schlagworteintrag \"" + entryName
				+ "\" ,seine Querverweise, seine Untereinträge und deren Querverweise wirklich löschen?";
	}

	@Override
	protected void deleteItem(AuthorElement itemNode) throws AuthorOperationException {

		String deleteID = itemNode.getAttribute("xml:id").getValue();

		deleteCrossReferences(deleteID);
		deleteChildren(itemNode);
		documentController.deleteNode(itemNode);
	}

	private void deleteChildren(AuthorElement itemNode) throws AuthorOperationException {

		// Die Liste der untergeordneten Schlagwörter durchlaufen und bei diesen
		// ebenfalls rekursiv die Löschen-Methode anwerden
		AuthorElement childItemList = XMLUtility.getFirstChildByLocalNameAndType(itemNode, "list", "Sachschlagwörter");

		if (childItemList != null) {
			AuthorElement[] childItems = childItemList.getElementsByLocalName("item");

			for (int i = 0; i < childItems.length; i++) {
				deleteItem(childItems[i]);
			}
		}
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "Kein Schlagwort zum Löschen gewählt."
				+ "\nBitte setzen sie den Textcursor innerhalb eines Schlagworteintrags und versuchen Sie es erneut.";
	}

	@Override
	protected AuthorElement nodeWithinEntry(AuthorElement node) {
		return RegisterUtility.nodeWithinKeyWordItemNode(node);
	}

	@Override
	protected String getEntryName(AuthorElement entryNode) throws BadLocationException {

		AuthorElement nameNode = XMLUtility.getFirstChildByLocalName(entryNode, "term");

		return nameNode.getTextContent();
	}

}
