package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.RegisterKeywordBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterKeywordNestedDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen
 * Sachschlagworteintrag in einer geschachtelten Sachschlagwort-RDB-Liste
 * bearbeiten können. Bietet die Möglichkeit den Begriff und die Querverweise
 * des Eintrags zu verändern.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterEditKeywordNested extends RegisterNewKeywordNested {

	private RegisterKeywordBean selectedKeywordBean;

	@Override
	public String getDescription() {
		return "Wenn der Textcursor sich innerhalb eines schon bestehenden Schlagworteintrags befindet, öfnet sich ein Dialogfenster. In diesem werden die aktuellen"
				+ "Daten und Verknüpfungen des Schlagworteintrags angezeigt und können verändert werden.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {
		initializeOxygenInterfaceVariables(authorAccess);

		if (!validateUrlArgument(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement itemNode;
			itemNode = RegisterUtility.nodeWithinKeyWordItemNode(surroundNode);

			if (itemNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			selectedKeywordBean = extractSelectedKeywordInfo(itemNode, documentController, oxygenFrame);

			ReadRegisterBasic registerReader = getRegisterEntries(args);
			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();

			RegisterKeywordNestedDialog dlg = new RegisterKeywordNestedDialog(oxygenFrame,
					"Schlagworteintrag bearbeiten", eintraege, ids);

			input = dlg.displayDialog(selectedKeywordBean);

			if (input.isFilled) {
				if (!input.keywordTerm.equalsIgnoreCase(selectedKeywordBean.keywordTerm)) {
					updateKeyWordTerm(itemNode, documentController);
				}
				updateCrossReferences(itemNode, documentController);
			}
		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			e.printStackTrace();
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	private RegisterKeywordBean extractSelectedKeywordInfo(AuthorElement node,
			AuthorDocumentController documentController, JFrame parentFrame)
			throws BadLocationException, AuthorOperationException {

		RegisterKeywordBean keywordBean = new RegisterKeywordBean();

		keywordBean.id = getSelectedKeywordId(node);
		keywordBean.keywordTerm = getSelectedKeywordTerm(node);
		keywordBean.setKeywordCrossReferences(reconstructKeywordCrossReferences(node, documentController, parentFrame));

		return keywordBean;
	}

	private String getSelectedKeywordId(AuthorElement node) {
		AttrValue idAttrVal = node.getAttribute("xml:id");

		return idAttrVal.getRawValue();
	}

	private String getSelectedKeywordTerm(AuthorElement node) throws BadLocationException {

		AuthorElement[] termNodes = node.getElementsByLocalName("term");
		String term = termNodes[0].getTextContent();

		return term;
	}

	private ArrayList<String[]> reconstructKeywordCrossReferences(AuthorElement node,
			AuthorDocumentController documentController, JFrame parentFrame)
			throws BadLocationException, AuthorOperationException {

		ArrayList<String[]> returnList = new ArrayList<String[]>();

		AuthorElement[] crossReferences = node.getElementsByLocalName("ref");

		String itemXPathBase = ("//*[local-name()='list' and @type='Sachschlagwörter']"
				+ "/*[local-name()='item' and @xml:id='");

		boolean referencesMissing = false;

		for (int i = 0; i < crossReferences.length; i++) {

			String crossReferenceId = getCrossReferenceId(crossReferences[i]);
			String referenceXPath = itemXPathBase + crossReferenceId + "']";

			AuthorNode[] referenceTarget = documentController.findNodesByXPath(referenceXPath, false, true, true);

			if (referenceTarget.length < 1) {
				referencesMissing = true;
			} else {

				String[] refIdAndValue = reconstructCrossReference((AuthorElement) referenceTarget[0]);
				returnList.add(refIdAndValue);

			}
		}

		if (referencesMissing) {
			JOptionPane.showMessageDialog(parentFrame,
					"Es konnten nicht alle Registerreferenzen wiederhergestellt werden.\n"
							+ "Womöglich wurden die referenzierten Einträge aus dem Register gelöscht.\n"
							+ "Durch das Abschließen der Operation werden diese verlorenen Verknüpfungen automatisch gelöscht.",
					"Fehlende Referenzen", JOptionPane.INFORMATION_MESSAGE);
		}

		return returnList;

	}

	private String[] reconstructCrossReference(AuthorElement refNode) throws BadLocationException {

		String[] reference = new String[2];

		AuthorElement[] refTerms = refNode.getElementsByLocalName("term");
		String refTerm = refTerms[0].getTextContent();

		reference[0] = refTerm;
		AttrValue idAttrVal = refNode.getAttribute("xml:id");
		reference[1] = idAttrVal.getRawValue();

		return reference;
	}

	private String getCrossReferenceId(AuthorElement refElement) {
		String crossReferenceId = XMLUtility.getAttributeRawValue(refElement, "target");

		return crossReferenceId;
	}

	private void updateKeyWordTerm(AuthorElement itemNode, AuthorDocumentController documentController)
			throws AuthorOperationException {
		AuthorElement[] termNodes = itemNode.getElementsByLocalName("term");
		AuthorElement termNode = termNodes[0];

		int termNodeStart = termNode.getStartOffset();
		int termNodeEnd = termNode.getEndOffset();

		String keywordTerm = XMLUtility.escapeXML(input.keywordTerm);
		String newTerm = "<term " + XMLUtility.TEIXMLNS + ">" + keywordTerm + "</term>";

		documentController.delete(termNodeStart, termNodeEnd);
		documentController.insertXMLFragment(newTerm, termNodeStart);

	}

	private void updateCrossReferences(AuthorElement itemNode, AuthorDocumentController documentController)
			throws AuthorOperationException {

		AuthorElement[] crossRefs = itemNode.getElementsByLocalName("ref");

		if (crossRefs.length > 0) {
			for (int i = crossRefs.length - 1; i > -1; i--) {

				documentController.deleteNode(crossRefs[i]);
			}
		}
		int itemEnd = itemNode.getEndOffset();

		String newCrossRefs = constructCrossReferenceFragment();

		documentController.insertXMLFragment(newCrossRefs, itemEnd);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "Kein Schlagwort zum bearbeiten gewählt!\n"
				+ "Bitte positionieren Sie den Textcursor innerhalb eines Sachschlagworteintrags, um die Operation auszuführen.";
	}

}
