package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.beans.SingleInputBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SingleInputDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation, mit der die Benutzer*innen einem RDB-Eintrag einen Weblink
 * (<ref @type="Weblink">-Element hinzufügen könnenn.
 * 
 * @author Maximilian Schrott
 * 
 */
public class RegisterInsertWeblink extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in das die Benutzer*innen eine URL eingeben könnnen."
				+ " Der Operation erzeugt daraus einen Weblink und fügt diesen dem Registerdatenbankeintrag an der aktuellen Position hinzu.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement insertionNode = findInsertionNode(surroundNode);

			if (insertionNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			SingleInputDialog dlg = new SingleInputDialog(oxygenFrame, "Neuen Weblink einfügen", "URL");

			SingleInputBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertWebLinkFragment(input, insertionNode);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void insertWebLinkFragment(SingleInputBean input, AuthorElement insertionNode)
			throws AuthorOperationException {
		String newWeblinkFragment = constructWeblinkFragment(input, insertionNode);

		int insertionOffset = insertionNode.getEndOffset();
		documentController.insertXMLFragment(newWeblinkFragment, insertionOffset);

	}

	private String constructWeblinkFragment(SingleInputBean input, AuthorElement insertionNode) {
		StringBuilder fragmentBuilder = new StringBuilder();
		String linkText = XMLUtility.escapeXML(input.inputText);

		fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Weblink' ");
		fragmentBuilder.append("target='" + linkText + "'/>");

		return fragmentBuilder.toString();
	}

	// Prüft ob sich der Textcursor innerhalb eines RDB-Eintrags befindet bzw.
	// bis zu zwei Ebenen unter diesem; Gibt Null zurück wenn nicht
	private AuthorElement findInsertionNode(AuthorElement surroundNode) {

		AuthorElement parentNode = XMLUtility.getNodeParentElement(surroundNode);
		AuthorElement grandParentNode = XMLUtility.getNodeParentElement(parentNode);

		if (nodeValidForWeblink(surroundNode)) {
			return surroundNode;

		} else if (nodeValidForWeblink(parentNode)) {
			return parentNode;

		} else if (nodeValidForWeblink(grandParentNode)) {
			return grandParentNode;
		} else {

			return null;
		}
	}

	private boolean nodeValidForWeblink(AuthorElement node) {
		return node != null && (nodeIsRegisterBibl(node) || nodeIsRegisterSource(node));
	}

	// FÜR GQL-, ZTG-, UGL-, und GAB
	private boolean nodeIsRegisterBibl(AuthorElement node) {
		AuthorElement parentNode = XMLUtility.getNodeParentElement(node);
		if (XMLUtility.elementDisplayNameEquals(node, "bibl")) {
			if (XMLUtility.elementDisplayNameEquals(parentNode, "listBibl")) {
				return true;
			}
		}
		return false;
	}

	// Für UGQ
	private boolean nodeIsRegisterSource(AuthorElement node) {
		AuthorElement parentNode = XMLUtility.getNodeParentElement(node);

		if (XMLUtility.elementDisplayNameEquals(node, "item")) {
			if (XMLUtility.elementIsListOfType(parentNode, "Quellen")) {
				return true;
			}
		}
		return false;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein Weblink eingefügt werden.\n"
				+ "Führen Sie die Operation bitte innerhalb eines RDB-Eintrages aus.";
	}

}
