package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.beans.SingleInputBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SingleInputDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen
 * Zeitungs-/Zeitschriften-Eintrag in die RDB einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewPeriodical extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen neuen Eintrag für das Register anlegen und einfügen können."
				+ "Der neue Titel wird an der Cursorposition in das aktuelle Dokument eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap arg1)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;
		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			if (!XMLUtility.elementDisplayNameEquals(surroundNode, "listBibl")
					|| !XMLUtility.elementAttributeEquals(surroundNode, "type", "Zeitungen")) {
				displayInvalidCaretPositionMessage();
				return;
			}

			SingleInputDialog dlg = new SingleInputDialog(oxygenFrame, "Neue Zeitung/Zeitschrift anlegen", "Name");

			SingleInputBean input = dlg.displayDialog();

			if (input.isFilled) {

				insertPeriodicalFragment(input);
			}

		} catch (

		BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	private void insertPeriodicalFragment(SingleInputBean input) throws AuthorOperationException {
		String newPeriodicalFragment = constructPeriodicalFragment(input);
		documentController.insertXMLFragment(newPeriodicalFragment, caretPosition);
	}

	private String constructPeriodicalFragment(SingleInputBean input) {

		String periodicalId = RegisterUtility.generateID(input.inputText, false, 20);
		String periodicalName = XMLUtility.escapeXML(input.inputText);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<bibl " + XMLUtility.TEIXMLNS + " xml:id='" + periodicalId + "'>");
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + " type='Name'>");
		fragmentBuilder.append(periodicalName + "</title>");
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + " type='Kürzel'/>");
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + " type='Zusatz'/>");
		fragmentBuilder.append("</bibl>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf keine Zeitung/Zeitschrift eingefügt werden.\n "
				+ "Bitte stellen Sie sicher, dass dich der Cursor innerhalb der Liste für Zeitungen/Zeitschriften befindet "
				+ "und nicht innerhalb eines anderen Registereintrags.";
	}

}
