package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.beans.SingleInputBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.SingleInputDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Orts-Eintrag in die RDB
 * einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewPlace extends OxyditionOperation {

	protected String getInvalidCaretPositionMessage() {
		return "An dieser Position darf kein Ortseintrag eingefügt werden.\n "
				+ "Bitte stellen Sie sicher, dass dich der Cursor innerhalb der Ortsliste befindet "
				+ "und nicht innerhalb eines anderen Ortseintrags.";
	}

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen einen neuen Ort für das Register anlegen und einfügen können."
				+ "Der neue Titel wird an der Cursorposition in das aktuelle Dokument eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap arg1)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			if (!nodeWithinPlaceList(surroundNode)) {
				displayInvalidCaretPositionMessage();
				return;
			}
			SingleInputBean input = new SingleInputBean();
			SingleInputDialog dlg = new SingleInputDialog(oxygenFrame, "Neuen Ort ins Register einfügen", "Ortsname");
			input = dlg.displayDialog();
			if (input.isFilled) {
				insertNewPlace(input);

			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	// Überprüft ob der Elternknoten des übergebenen Knoten eine
	// <listBibl>-Element mit @type="Literatur" ist
	private boolean nodeWithinPlaceList(AuthorElement checkNode) {

		if (XMLUtility.elementDisplayNameEquals(checkNode, "listPlace")) {
			return true;
		}

		return false;

	}

	private void insertNewPlace(SingleInputBean input) throws AuthorOperationException {
		String newPlace = constructNewPlaceXMLFragment(input);
		documentController.insertXMLFragment(newPlace, caretPosition);
	}

	private String constructNewPlaceXMLFragment(SingleInputBean input) {

		String xmlid = RegisterUtility.generateID(input.inputText, false, 20);
		String placeName = XMLUtility.escapeXML(input.inputText);

		StringBuilder fragmentBuilder = new StringBuilder();

		fragmentBuilder.append("<place " + XMLUtility.TEIXMLNS + " xml:id='" + xmlid + "'>");
		fragmentBuilder.append("<placeName " + XMLUtility.TEIXMLNS + ">" + placeName + "</placeName>");
		fragmentBuilder.append(
				"<location " + XMLUtility.TEIXMLNS + " type='Punkt'><geo " + XMLUtility.TEIXMLNS + "/></location>");
		fragmentBuilder.append("<idno type='gnd'/>");
		fragmentBuilder.append("</place>");

		return fragmentBuilder.toString();
	}

}
